# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.xml_sped.base import TagCaracter
from .base import XMLMDFe, NAMESPACE_MDFE
from . import ESQUEMA_ATUAL
import os


DIRNAME = os.path.dirname(__file__)


class InfDoc(XMLMDFe):
    """Informacoes dos documentos que serao inseridos no MDF-e."""
    def __init__(self):
        super(InfDoc, self).__init__()
        self.cMunDescarga = TagCaracter(nome='cMunDescarga', tamanho=[7, 7], raiz='//infDoc')
        self.xMunDescarga = TagCaracter(nome='xMunDescarga', tamanho=[2, 60], raiz='//infDoc')
        self.chNFe = TagCaracter(nome='chNFe', tamanho=[44, 44], raiz='//infDoc')

    @property
    def xml(self):
        xml = '<infDoc>'
        xml += self.cMunDescarga.xml
        xml += self.xMunDescarga.xml
        xml += self.chNFe.xml
        xml += '</infDoc>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cMunDescarga.xml = arquivo
        self.xMunDescarga.xml = arquivo
        self.chNFe.xml = arquivo


class EvIncDFeMDFe(XMLMDFe):
    """Leiaute especifico do evento de inclusao de DF-e."""
    def __init__(self):
        super(EvIncDFeMDFe, self).__init__()
        self.descEvento = TagCaracter(
            nome='descEvento', tamanho=[13, 13], raiz='/',
            valor='Inclusao DF-e')
        self.nProt = TagCaracter(nome='nProt', tamanho=[15, 15], raiz='//evIncDFeMDFe')
        self.cMunCarrega = TagCaracter(nome='cMunCarrega', tamanho=[7, 7], raiz='//evIncDFeMDFe')
        self.xMunCarrega = TagCaracter(nome='xMunCarrega', tamanho=[2, 60], raiz='//evIncDFeMDFe')
        self.infDoc = []
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'evInclusaoDFeMDFe_v3.00.xsd'

    @property
    def xml(self):
        # xml = XMLMDFe.xml.fget()
        xml = ''
        if self.cMunCarrega.valor:
            xml += '<evIncDFeMDFe xmlns="%s">' % NAMESPACE_MDFE
            xml += self.descEvento.xml
            xml += self.nProt.xml
            xml += self.cMunCarrega.xml
            xml += self.xMunCarrega.xml
            for infdoc in self.infDoc:
                xml += infdoc.xml
            xml += '</evIncDFeMDFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.descEvento.xml = arquivo
            self.nProt.xml = arquivo
            self.cMunCarrega.xml = arquivo
            self.xMunCarrega.xml = arquivo
            self.infDoc = self.le_grupo('//evIncDFeMDFe/infDoc', InfDoc)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
