# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.xml_sped.base import TagCaracter, TagInteiro, TagDecimal, TagDataHoraUTC
from pynfe.xml_sped.assinatura import Signature
from .evIncCondutorMDFe import EvIncCondutorMDFe
from .evInclusaoDFeMDFe import EvIncDFeMDFe
from pynfe.xml_sped.base import ABERTURA
from .base import XMLMDFe, NAMESPACE_MDFE
from .evCancMDFe import EvCancMDFe
from .evEncMDFe import EvEncMDFe
from . import ESQUEMA_ATUAL
import os


DIRNAME = os.path.dirname(__file__)


class DetEvento(XMLMDFe):
    """Informacoes do evento específico."""
    def __init__(self):
        super(DetEvento, self).__init__()
        self.versao = TagDecimal(
            nome='detEvento', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade='versaoEvento',
            raiz='//infEvento', namespace=NAMESPACE_MDFE, valor='3.00')
        self.eventoCancelamento = EvCancMDFe()
        self.eventoEncerramento = EvEncMDFe()
        self.eventoIncCondutor = EvIncCondutorMDFe()
        self.eventoIncDFe = EvIncDFeMDFe()
        self.evento = None

    @property
    def xml(self):
        xml = '<detEvento versaoEvento="%s">' % self.versao.valor
        xml += self.eventoCancelamento.xml if self.eventoCancelamento.xJust.valor else u''
        xml += self.eventoEncerramento.xml if self.eventoEncerramento.dtEnc.valor else u''
        xml += self.eventoIncCondutor.xml if self.eventoIncCondutor.condutor.cpf.valor else u''
        xml += self.eventoIncDFe.xml if self.eventoIncDFe.cMunCarrega.valor else u''
        xml += '</detEvento>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.eventoCancelamento.xml = arquivo
        self.eventoEncerramento.xml = arquivo
        self.eventoIncCondutor.xml = arquivo
        self.eventoIncDFe.xml = arquivo


class InfEvento(XMLMDFe):
    """Grupo de informacoes do registro do evento."""
    def __init__(self):
        super(InfEvento, self).__init__()
        self.Id = TagCaracter(nome='infEvento', tamnho=[54, 54], propriedade='Id', raiz='/')
        self.cOrgao = TagInteiro(nome='cOrgao', tamanho=[2, 2], raiz='//infEvento')
        self.tpAmb = TagInteiro(nome='tpAmb', tamanho=[1, 1], raiz='//infEvento')
        self.cnpj = TagCaracter(nome='CNPJ', tamanho=[14, 14], raiz='//infEvento')
        self.chMDFe = TagCaracter(nome='chMDFe', tamanho=[44, 44], raiz='//infEvento')
        self.dhEvento = TagDataHoraUTC(nome='dhEvento', raiz='//infEvento')
        self.tpEvento = TagInteiro(nome='tpEvento', tamanho=[6, 6], raiz='//infEvento')
        self.nSeqEvento = TagInteiro(nome='nSeqEvento', tamanho=[1, 2], raiz='//infEvento')
        self.detEvento = DetEvento()

    @property
    def xml(self):
        self.Id.valor = 'ID%s%s%02d' % (self.tpEvento.valor, self.chMDFe.valor, self.nSeqEvento.valor)
        xml = '<infEvento Id="%s">' % self.Id.valor
        xml += self.cOrgao.xml
        xml += self.tpAmb.xml
        xml += self.cnpj.xml
        xml += self.chMDFe.xml
        xml += self.dhEvento.xml
        xml += self.tpEvento.xml
        xml += self.nSeqEvento.xml
        xml += self.detEvento.xml
        xml += '</infEvento>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.Id.xml = arquivo
        self.cOrgao.xml = arquivo
        self.tpAmb.xml = arquivo
        self.cnpj.xml = arquivo
        self.chMDFe.xml = arquivo
        self.dhEvento.xml = arquivo
        self.tpEvento.xml = arquivo
        self.nSeqEvento.xml = arquivo
        self.detEvento.xml = arquivo


class EventoMDFe(XMLMDFe):
    """Mensagem de solicitacao de registro de evento do MDF-e."""
    def __init__(self):
        super(EventoMDFe, self).__init__()
        self.versao = TagDecimal(
            nome='eventoMDFe', tamanho=[1, 4], decimais=[0, 2, 2], propriedade='versao',
            raiz='/', namespace=NAMESPACE_MDFE, valor='3.00')
        self.infEvento = InfEvento()
        self.signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'eventoMDFe_v3.00.xsd'

    @property
    def xml(self):
        xml = '%s<eventoMDFe xmlns="%s" versao="%s">' % (str(ABERTURA), NAMESPACE_MDFE, self.versao.valor)
        xml += self.infEvento.xml
        self.signature.URI = '#' + self.infEvento.Id.valor
        xml += self.signature.xml
        xml += '</eventoMDFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.infEvento.xml = arquivo
        self.signature.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
