# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from .webservices import WEBSERVICE, MDFE_CONSULTA, MDFE_RECEPCAO, MDFE_CONS_NAO_ENC
from .webservices import MDFE_STATUS_SERVICO, MDFE_RET_RECEPCAO, MDFE_RECEPCAO_EVENTO
from pynfe.xml_sped.certificado import Certificado
from .evInclusaoDFeMDFe import EvIncDFeMDFe, InfDoc
from .retConsStatServ import RetConsStatServMDFe
from .retConsMDFeNaoEnc import RetConsMDFeNaoEnc
from .evIncCondutorMDFe import EvIncCondutorMDFe
from pynfe.xml_sped.base import remove_accents
from .retConsReciMDFe import RetConsReciMDFe
from .retConsSitMDFe import RetConsSitMDFe
from .consStatServ import ConsStatServMDFe
from .consMDFeNaoEnc import ConsMDFeNaoEnc
from pynfe.xml_sped.base import ABERTURA
from .retEventoMDFe import RetEventoMDFe
from .soap import SOAPEnvio, SOAPRetorno
from .consReciMDFe import ConsReciMDFe
from .retEnviMDFe import RetEnviMDFe
from .consSitMDFe import ConsSitMDFe
from .eventoMDFe import EventoMDFe
from .evCancMDFe import EvCancMDFe
from .evEncMDFe import EvEncMDFe
from .enviMDFe import EnviMDFe
from datetime import datetime
import requests
import tempfile
import logging
import pytz
import time
import os


PEM = os.path.dirname(os.path.abspath(__file__)) + '/../../xml_sped/cadeia.pem'


class CertificadoMDFe(Certificado):
    def __init__(self, cert=None, key=None):
        super(CertificadoMDFe, self).__init__()
        self.certificado = cert
        self.chave = key

    def _prepara_doc_xml(self, xml):
        # Determina o tipo de arquivo que vai ser assinado, procurando pela tag correspondente
        if 'infMDFe' in xml:
            doctype = '<!DOCTYPE MDFe [<!ATTLIST infMDFe Id ID #IMPLIED>]>'
        elif 'infEvento' in xml:
            doctype = '<!DOCTYPE eventoMDFe [<!ATTLIST infEvento Id ID #IMPLIED>]>'
        else:
            raise ValueError('Tipo de arquivo desconhecido para assinatura/validacao')

        xml = xml.replace(ABERTURA, ABERTURA + doctype)
        xml = xml.replace('\n', '')
        return xml

    def _finaliza_xml(self, xml):
        # Determina o tipo de arquivo que vai ser assinado, procurando pela tag correspondente
        if 'infMDFe' in xml:
            doctype = '<!DOCTYPE MDFe [<!ATTLIST infMDFe Id ID #IMPLIED>]>'
        elif 'infEvento' in xml:
            doctype = '<!DOCTYPE envEvento [<!ATTLIST infEvento Id ID #IMPLIED>]>'
        else:
            raise ValueError('Tipo de arquivo desconhecido para assinatura/validacao')

        xml = xml.replace(ABERTURA + doctype, ABERTURA)
        xml = xml.replace('\n', '')
        return xml


class ProcessoMDFe(object):
    def __init__(self, servico, envio, resposta):
        self.servico = servico
        self.envio = envio
        self.resposta = resposta

    def __str__(self):
        result = ''
        if hasattr(self.resposta, 'cStat') and self.resposta.cStat.valor:
            motivo = self.resposta.xMotivo.valor
            result = '[%s] %s' % (self.resposta.cStat.valor, motivo)
        elif hasattr(self.resposta, 'infEvento'):
            if hasattr(self.resposta.infEvento, 'cStat') and self.resposta.infEvento.cStat.valor:
                motivo = self.resposta.infEvento.xMotivo.valor
                result = '[%s] %s' % (self.resposta.infEvento.cStat.valor, motivo)
        if hasattr(self.resposta, 'protMDFe'):
            if hasattr(self.resposta.protMDFe, 'infProt') and result:
                result += ' %s@' % self.resposta.protMDFe.infProt.chMDFe.valor
                result += '%s #' % self.resposta.protMDFe.infProt.dhRecbto.valor
                result += '%s' % self.resposta.protMDFe.infProt.nProt.valor
        return result


class ProcessadorMDFe(object):
    logger = logging.getLogger('MDFe')  # Registro de log da classe

    def __init__(self, cert, chave, ambiente, estado):
        """
        Metodo construtor da classe
        :param cert: certificado digital da empresa
        :param chave: chave privada para o uso do certificado
        :param ambiente: ambiente de atuacao da MDFe: 1=producao ou 2=homologacao
        :param estado: codigo IBGE do estado (UF)
        """
        self._cert = cert
        self._chave = chave
        self._ambiente = ambiente
        self._estado = estado
        self.tentativas = 5

    def conectar(self, servico, envio, resposta):
        """Conexao com o webservice"""
        # Tratamento do certificado digital e da chave privada
        cert = tempfile.NamedTemporaryFile('w', dir='/dev/shm')
        chave = tempfile.NamedTemporaryFile('w', dir='/dev/shm')
        cert.write(self._cert)
        cert.flush()
        chave.write(self._chave)
        chave.flush()
        _cert = cert.name, chave.name

        # Le parametros do webservice
        ws = WEBSERVICE[self._estado][self._ambiente][servico]

        # Dados de envio
        body = SOAPEnvio()
        body.servico = ws['servico']
        body.cUF = self._estado
        body.dados = envio
        body = body.xml
        content_type = 'application/soap+xml; charset=utf-8; ' \
                       'action="http://www.portalfiscal.inf.br/mdfe/wsdl/%s"'
        header = {'content-type': content_type % ws['servico']}

        # Dados de retorno
        soap_retorno = SOAPRetorno()
        soap_retorno.servico = ws['servico']
        soap_retorno.metodo = ws['metodo']
        soap_retorno.resposta = resposta
        self.logger.info('>> Metodo da requisicao: POST')
        self.logger.info('>> XML envio: %s' % body)
        # Conectar servico
        try:
            for t in range(1, self.tentativas + 1):
                self.logger.info('<<< Tentativa [%d] >>>' % (t))
                self.logger.info('-->> Conectando com a SEFAZ...')
                result = requests.post(
                    url=ws['url'], data=body, cert=_cert, verify=PEM, headers=header)
                if result.ok:
                    soap_retorno.resposta.xml = result.content
                    self.logger.info('<< XML resposta: %s' % soap_retorno.resposta.xml)
                    break
                else:
                    resposta.alertas.append((t, '%s-%s' % (result.status_code, result.reason)))
                    time.sleep(1)
        except Exception as err:
            resposta.alertas.append(err.__str__())
        finally:
            cert.close()
            chave.close()

    def consultar_servico(self):
        """Consulta do status do Servico MDF-e."""
        envio = ConsStatServMDFe()
        resposta = RetConsStatServMDFe()
        self.logger.info('*** consultar servico ***')
        processo = ProcessoMDFe(MDFE_STATUS_SERVICO, envio, resposta)
        envio.tpAmb.valor = self._ambiente
        envio.validar()  # Try, if error append process alerts
        self.conectar(MDFE_STATUS_SERVICO, envio, resposta)
        return processo

    def enviar_lote(self, lote):
        """Envio de lote de MDF-e."""
        envio = EnviMDFe()
        resposta = RetEnviMDFe()
        cert_mdfe = CertificadoMDFe()
        cert_mdfe.chave = self._chave
        cert_mdfe.certificado = self._cert
        lote = lote if isinstance(lote, (list, tuple)) else [lote]
        for mdfe in lote:
            mdfe.signature.xml = cert_mdfe.assina_xml(mdfe.xml)
            mdfe.validar()
        envio.mdfe = lote
        envio.idLote.valor = "{0}{1}".format(
            str(self._ambiente), datetime.now().strftime('%Y%m%d%H%M%S'))
        envio.validar()
        processo = ProcessoMDFe(MDFE_STATUS_SERVICO, envio, resposta)
        self.logger.info('*** enviar lote ***')
        self.conectar(MDFE_RECEPCAO, envio, resposta)
        return processo

    def consultar_recibo(self, recibo):
        """Devolve o resultado do processamento do MDF-e. (envio de lote)"""
        envio = ConsReciMDFe()
        resposta = RetConsReciMDFe()
        envio.nRec.valor = recibo
        envio.tpAmb.valor = self._ambiente
        envio.validar()
        processo = ProcessoMDFe(MDFE_RET_RECEPCAO, envio, resposta)
        self.logger.info('*** consultar recibo ***')
        self.conectar(MDFE_RET_RECEPCAO, envio, resposta)
        return processo

    def consultar_mdfe(self, chave):
        """Devolve o resultado da consulta a chave do MDF-e"""
        envio = ConsSitMDFe()
        resposta = RetConsSitMDFe()
        envio.chMDFe.valor = chave
        envio.tpAmb.valor = self._ambiente
        envio.validar()
        processo = ProcessoMDFe(MDFE_CONSULTA, envio, resposta)
        self.logger.info('*** consultar mdfe (chave) ***')
        self.conectar(MDFE_CONSULTA, envio, resposta)
        return processo

    def consultar_nao_encerrado(self, cpf=False, cnpj=False):
        """Consulta MDF-e nao encerrados na base de dados do ambiente autorizador"""
        envio = ConsMDFeNaoEnc()
        resposta = RetConsMDFeNaoEnc()
        envio.tpAmb.valor = self._ambiente
        cpf = cpf and str(cpf).isalnum() and str(int(cpf)).zfill(11) or ''
        cnpj = cnpj and str(cnpj).isalnum() and str(int(cnpj)).zfill(14) or ''
        envio.cpf.valor = cpf
        envio.cnpj.valor = cnpj
        envio.validar()
        processo = ProcessoMDFe(MDFE_CONS_NAO_ENC, envio, resposta)
        self.logger.info('*** consultar mdfe nao encerrado ***')
        self.conectar(MDFE_CONS_NAO_ENC, envio, resposta)
        return processo

    def incluir_condutor(self, chave, cpf, nome, sequencia=1):
        hoje = datetime.utcnow()
        hoje = hoje.replace(tzinfo=pytz.utc)
        hoje = datetime.astimezone(hoje, pytz.timezone("Brazil/East"))
        envio = EventoMDFe()
        envio.infEvento.tpEvento.valor = 110114
        envio.infEvento.tpAmb.valor = self._ambiente
        envio.infEvento.nSeqEvento.valor = sequencia
        envio.infEvento.cnpj.valor = chave[6:20]
        envio.infEvento.cOrgao.valor = self._estado
        envio.infEvento.chMDFe.valor = chave
        evento = EvIncCondutorMDFe()
        evento.condutor.cpf.valor = cpf
        evento.condutor.xNome.valor = nome
        evento.validar()
        envio.infEvento.dhEvento.valor = datetime.strftime(
            hoje, '%Y-%m-%dT%H:%M:%S') + str(hoje)[-6:]
        envio.infEvento.detEvento.eventoIncCondutor = evento
        envio.signature.xml = CertificadoMDFe(self._cert, self._chave).assina_xml(envio.xml)
        envio.validar()
        resposta = RetEventoMDFe()
        processo = ProcessoMDFe(MDFE_RECEPCAO_EVENTO, envio, resposta)
        self.logger.info('*** evento incluir condutor ***')
        self.conectar(MDFE_RECEPCAO_EVENTO, envio, resposta)
        return processo

    def incluir_dfe(self, chave, protocolo, cod_municipio, nome_municipio, inf_doc, sequencia=1):
        """Carregamento posterior"""
        hoje = datetime.utcnow()
        hoje = hoje.replace(tzinfo=pytz.utc)
        hoje = datetime.astimezone(hoje, pytz.timezone("Brazil/East"))
        envio = EventoMDFe()
        envio.infEvento.tpEvento.valor = 110115
        envio.infEvento.tpAmb.valor = self._ambiente
        envio.infEvento.nSeqEvento.valor = sequencia
        envio.infEvento.cnpj.valor = chave[6:20]
        envio.infEvento.cOrgao.valor = self._estado
        envio.infEvento.chMDFe.valor = chave
        evento = EvIncDFeMDFe()
        evento.nProt.valor = protocolo
        evento.cMunCarrega.valor = cod_municipio
        evento.xMunCarrega.valor = nome_municipio
        for doc in inf_doc if isinstance(inf_doc, (list, tuple)) else [inf_doc]:
            d = InfDoc()
            d.cMunDescarga.valor = doc['codigo']
            d.xMunDescarga.valor = remove_accents(doc['municipio'])
            d.chNFe.valor = doc['nfe']
            evento.infDoc.append(d)
        evento.validar()
        envio.infEvento.dhEvento.valor = datetime.strftime(
            hoje, '%Y-%m-%dT%H:%M:%S') + str(hoje)[-6:]
        envio.infEvento.detEvento.eventoIncDFe = evento
        envio.signature.xml = CertificadoMDFe(self._cert, self._chave).assina_xml(envio.xml)
        envio.validar()
        resposta = RetEventoMDFe()
        processo = ProcessoMDFe(MDFE_RECEPCAO_EVENTO, envio, resposta)
        self.logger.info('*** evento incluir condutor ***')
        self.conectar(MDFE_RECEPCAO_EVENTO, envio, resposta)
        return processo

    def encerrar_mdfe(self, chave, protocolo, cod_uf, cod_municipio):
        """Solicitacao de encerramento do MDF-e"""
        hoje = datetime.utcnow()
        hoje = hoje.replace(tzinfo=pytz.utc)
        hoje = datetime.astimezone(hoje, pytz.timezone("Brazil/East"))
        envio = EventoMDFe()
        envio.infEvento.tpEvento.valor = 110112
        envio.infEvento.tpAmb.valor = self._ambiente
        envio.infEvento.nSeqEvento.valor = 1  # 1 --> Somente um encerramento ;)
        envio.infEvento.cnpj.valor = chave[6:20]
        envio.infEvento.cOrgao.valor = self._estado
        envio.infEvento.chMDFe.valor = chave
        evento = EvEncMDFe()
        evento.nProt.valor = protocolo
        evento.dtEnc.valor = datetime.strftime(hoje, '%Y-%m-%d')
        evento.cUF.valor = cod_uf
        evento.cMun.valor = cod_municipio
        evento.validar()
        envio.infEvento.dhEvento.valor = datetime.strftime(
            hoje, '%Y-%m-%dT%H:%M:%S') + str(hoje)[-6:]
        envio.infEvento.detEvento.eventoEncerramento = evento
        envio.signature.xml = CertificadoMDFe(self._cert, self._chave).assina_xml(envio.xml)
        envio.validar()
        resposta = RetEventoMDFe()
        processo = ProcessoMDFe(MDFE_RECEPCAO_EVENTO, envio, resposta)
        self.logger.info('*** evento encerrar mdfe ***')
        self.conectar(MDFE_RECEPCAO_EVENTO, envio, resposta)
        return processo

    def cancelar_mdfe(self, chave, protocolo, justificativa):
        """Solicitacao de cancelamento do MDF-e"""
        hoje = datetime.utcnow()
        hoje = hoje.replace(tzinfo=pytz.utc)
        hoje = datetime.astimezone(hoje, pytz.timezone("Brazil/East"))
        envio = EventoMDFe()
        envio.infEvento.tpEvento.valor = 110111
        envio.infEvento.tpAmb.valor = self._ambiente
        envio.infEvento.nSeqEvento.valor = 1  # --> Somente um cancelamento ;)
        envio.infEvento.cnpj.valor = chave[6:20]
        envio.infEvento.cOrgao.valor = self._estado
        envio.infEvento.chMDFe.valor = chave
        evento = EvCancMDFe()
        evento.nProt.valor = protocolo
        evento.xJust.valor = justificativa
        evento.validar()
        envio.infEvento.dhEvento.valor = datetime.strftime(
            hoje, '%Y-%m-%dT%H:%M:%S') + str(hoje)[-6:]
        envio.infEvento.detEvento.eventoCancelamento = evento
        envio.signature.xml = CertificadoMDFe(self._cert, self._chave).assina_xml(envio.xml)
        envio.validar()
        resposta = RetEventoMDFe()
        processo = ProcessoMDFe(MDFE_RECEPCAO_EVENTO, envio, resposta)
        self.logger.info('*** evento encerrar mdfe ***')
        self.conectar(MDFE_RECEPCAO_EVENTO, envio, resposta)
        return processo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
