# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.xml_sped.base import TagInteiro, TagDecimal, ABERTURA
from . import NAMESPACE_MDFE
from .base import XMLMDFe


class MDFeCabecMsg(XMLMDFe):
    """Cabecalho (header) de mensagem de requisicao padrao SOAP."""
    def __init__(self):
        super(MDFeCabecMsg, self).__init__()
        self.servico = u''
        self.cUF = TagInteiro(
            nome='cUF', tamanho=[2, 2], raiz='//mdfeCabecMsg', valor=31)
        self.versaoDados = TagDecimal(
            nome='versaoDados', tamanho=[1, 4], raiz='//mdfeCabecMsg', valor='3.00')

    @property
    def xml(self):
        xml = '<mdfeCabecMsg xmlns="%s/wsdl/%s">' % (NAMESPACE_MDFE, self.servico)
        xml += self.cUF.xml
        xml += self.versaoDados.xml
        xml += '</mdfeCabecMsg>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cUF.xml = arquivo
        self.versaoDados.xml = arquivo


class MDFeDadosMsg(XMLMDFe):
    """Dados (data) de mensagem de requisicao padrao SOAP."""
    def __init__(self):
        super(MDFeDadosMsg, self).__init__()
        self.servico = u''
        self.dados = None

    @property
    def xml(self):
        xml = '<mdfeDadosMsg xmlns="%s/wsdl/%s">' % (NAMESPACE_MDFE, self.servico)
        xml += self.dados.raw_xml()
        xml += '</mdfeDadosMsg>'

        return xml


class SOAPEnvio(XMLMDFe):
    def __init__(self):
        super(SOAPEnvio, self).__init__()
        self.servico = u''
        self.cUF = None
        self.dados = None
        self.mdfeCabecMsg = MDFeCabecMsg()
        self.mdfeDadosMsg = MDFeDadosMsg()
        self._header = {'content-type': 'application/soap+xml; charset=utf-8'}

    @property
    def xml(self):
        self.mdfeCabecMsg.servico = self.servico
        self.mdfeCabecMsg.cUF.valor = self.cUF
        self.mdfeCabecMsg.versaoDados.valor = self.dados.versao.valor
        self.mdfeDadosMsg.servico = self.servico
        self.mdfeDadosMsg.dados = self.dados
        self._header['content-type'] += '; action="%s/wsdl/%s"' % (NAMESPACE_MDFE, self.servico)

        xml = ABERTURA
        xml += '<soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope">'
        xml += '<soap:Header>'
        xml += self.mdfeCabecMsg.xml
        xml += '</soap:Header>'
        xml += '<soap:Body>'
        xml += self.mdfeDadosMsg.xml
        xml += '</soap:Body>'
        xml += '</soap:Envelope>'

        return xml

    @property
    def header(self):
        header = self._header

        return header


class SOAPRetorno(XMLMDFe):
    def __init__(self):
        super(SOAPRetorno, self).__init__()
        self.servico = u''
        self.metodo = u''
        self.mdfeCabecMsg = MDFeCabecMsg()
        self.resposta = None

    @property
    def xml(self):
        xml = ABERTURA
        xml += '<soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope">'
        xml += '<soap:Header>'
        xml += '<mdfeCabecMsg xmlns="%s/wsdl/%s">' % (NAMESPACE_MDFE, self.servico)
        xml += self.mdfeCabecMsg.xml
        xml += '</mdfeCabecMsg>'
        xml += '</soap:Header>'
        xml += '<soap:Body>'
        xml += '<%sResult xmlns="%s/wsdl/%s">' % (self.metodo, NAMESPACE_MDFE, self.servico)
        xml += self.resposta.xml
        xml += '</%sResult>' % self.metodo
        xml += '</soap:Body>'
        xml += '</soap:Envelope>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.mdfeCabecMsg.xml = arquivo
        self.resposta.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
