# -*- coding: utf-8 -*-


from pynfe.xml_sped.base import *
from . import ESQUEMA_ATUAL
import os


DIRNAME = os.path.dirname(__file__)



class ConsNFeDest(XMLNFe):
    def __init__(self):
        super(ConsNFeDest, self).__init__()
        self.versao = TagDecimal(nome='consNFeDest', codigo='IP01', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.01', raiz='/')
        self.tpAmb = TagInteiro(nome='tpAmb', codigo='IP03', tamanho=[ 1, 1, 1], raiz='//consNFeDest')
        self.xServ =  TagCaracter(nome='xServ', codigo='IP04', tamanho=[18, 18], raiz='//consNFeDest', valor='CONSULTAR NFE DEST')
        self.CNPJ = TagCaracter(nome='CNPJ', codigo='IP05', tamanho=[ 0, 14], raiz='//consNFeDest')
        self.indNFe = TagInteiro(nome='indNFe', codigo='IP06', tamanho=[1, 1, 1], raiz='//consNFeDest')
        self.indEmi = TagInteiro(nome='indEmi', codigo='IP07', tamanho=[1, 1, 1], raiz='//consNFeDest')
        self.ultNSU = TagCaracter(nome='ultNSU', codigo='IP08', tamanho=[1, 15], raiz='//consNFeDest')
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'consNFeDest_v1.01.xsd'


    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.xServ.xml
        xml += self.CNPJ.xml
        xml += self.indNFe.xml
        xml += self.indEmi.xml
        xml += self.ultNSU.xml

        xml += '</consNFeDest>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml = arquivo
            self.xServ.xml = arquivo
            self.CNPJ.xml = arquivo
            self.indNFe.xml = arquivo
            self.indEmi.xml = arquivo
            self.ultNSU.xml = arquivo




class ResCCe(XMLNFe):
    def __init__(self):
        super(ResCCe, self).__init__()
        self.NSU = TagInteiro(nome='resCCe', codigo='IR39', tamanho=[1, 15], raiz='//ret', propriedade='NSU',obrigatorio=False)
        self.chNFe = TagCaracter(nome='chNFe', codigo='IR41', tamanho=[44, 44, 44], raiz='//ret/resCCe',obrigatorio=False)
        self.dhEvento = TagCaracter(nome='dhEvento', codigo='IR42', tamanho=[ 0, 30],raiz='//ret/resCCe',obrigatorio=False)
        self.tpEvento = TagCaracter(nome='tpEvento', codigo='IR43', tamanho=[6, 6, 6], raiz='//ret/resCCe',obrigatorio=False)
        self.nSeqEvento = TagInteiro(nome='nSeqEvento', codigo='IR44', tamanho=[2,2], raiz='//ret/resCCe',obrigatorio=False)
        self.descEvento = TagCaracter(nome='descEvento', codigo='IR45', tamanho=[ 5, 60], raiz='//ret/resCCe',obrigatorio=False)
        self.xCorrecao = TagCaracter(nome='descEvento', codigo='IR46', tamanho=[ 15, 1000], raiz='//ret/resCCe', obrigatorio=False)
        self.tpNF = TagInteiro(nome='tpNF', codigo='IR47', tamanho=[1, 1, 1], raiz='//ret/resCCe',obrigatorio=False)
        self.dhRecbto = TagCaracter(nome='dhRecbto', codigo='IR48', tamanho=[ 0, 30],raiz='//ret/resCCe',obrigatorio=False)

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        if self.NSU.xml:
            xml += '<ret>'
        xml += self.NSU.xml
        xml += self.chNFe.xml
        xml += self.dhEvento.xml
        xml += self.tpEvento.xml
        xml += self.nSeqEvento.xml
        xml += self.descEvento.xml
        xml += self.xCorrecao.xml
        xml += self.tpNF.xml
        xml += self.dhRecbto.xml
        if self.NSU.xml:
            xml += '</resCCe>'
#            xml += '</ret>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NSU.xml = arquivo
            self.chNFe.xml = arquivo
            self.dhEvento.xml = arquivo
            self.tpEvento.xml = arquivo
            self.nSeqEvento.xml = arquivo
            self.descEvento.xml = arquivo
            self.xCorrecao.xml = arquivo
            self.tpNF.xml = arquivo
            self.dhRecbto.xml = arquivo



class ResCanc(XMLNFe):
    def __init__(self):
        super(ResCanc, self).__init__()
        self.NSU = TagInteiro(nome='resCanc', codigo='IR25', tamanho=[1, 15], raiz='//ret', propriedade='NSU', obrigatorio=False)
        self.chNFe = TagCaracter(nome='chNFe', codigo='IR27', tamanho=[44, 44, 44], raiz='//ret/resCanc',obrigatorio=False)
        self.CNPJ = TagCaracter(nome='CNPJ', codigo='IR28', tamanho=[ 0, 14], raiz='//ret/resCanc',obrigatorio=False)
        self.CPF = TagCaracter(nome='CPF', codigo='IR29', tamanho=[11, 11], raiz='//ret/resCanc',obrigatorio=False)
        self.xNome = TagCaracter(nome='xNome', codigo='IR30', tamanho=[ 3, 60], raiz='//ret/resCanc',obrigatorio=False)
        self.IE = TagCaracter(nome='IE', codigo='IR31', tamanho=[ 1, 14], raiz='//ret/resCanc',obrigatorio=False)
        self.dEmi = TagCaracter(nome='dEmi', codigo='IR32', tamanho=[ 0, 30],raiz='//ret/resCanc',obrigatorio=False)
        self.tpNF = TagInteiro(nome='tpNF', codigo='IR33', tamanho=[1, 1, 1], raiz='//ret/resCanc',obrigatorio=False)
        self.vNF = TagDecimal(nome='vNF', codigo='IR34', tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='//ret/resCanc',obrigatorio=False)
        self.digVal = TagCaracter(nome='digVal', codigo='IR35', tamanho=[28, 28], raiz='//ret/resCanc',obrigatorio=False)
        self.dhRecbto = TagCaracter(nome='dhRecbto', codigo='IR36', tamanho=[ 0, 30],raiz='//ret/resCanc',obrigatorio=False)
        self.cSitNFe = TagInteiro(nome='cSitNFe', codigo='IR37', tamanho=[1, 1, 1], raiz='//ret/resCanc',obrigatorio=False)
        self.cSitConf = TagInteiro(nome='cSitConf', codigo='IR38', tamanho=[1, 1, 1], raiz='//ret/resCanc',obrigatorio=False)


    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        if self.NSU.xml:
            xml += '<ret>'
        xml += self.NSU.xml
        xml += self.chNFe.xml
        xml += self.CNPJ.xml
        xml += self.CPF.xml
        xml += self.xNome.xml
        xml += self.IE.xml
        xml += self.dEmi.xml
        xml += self.tpNF.xml
        xml += self.vNF.xml
        xml += self.digVal.xml
        xml += self.dhRecbto.xml
        xml += self.cSitNFe.xml
        xml += self.cSitConf.xml
        if self.NSU.xml:
            xml += '</resCanc>'
#            xml += '</ret>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NSU.xml = arquivo
            self.chNFe.xml = arquivo
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo
            self.xNome.xml = arquivo
            self.IE.xml = arquivo
            self.dEmi.xml = arquivo
            self.tpNF.xml = arquivo
            self.vNF.xml = arquivo
            self.digVal.xml = arquivo
            self.dhRecbto.xml = arquivo
            self.cSitNFe.xml = arquivo
            self.cSitConf.xml = arquivo

class ResNFe(XMLNFe):
    def __init__(self):
        super(ResNFe, self).__init__()
        self.NSU = TagInteiro(nome='resNFe', codigo='IR11', tamanho=[1, 15], raiz='//ret', propriedade='NSU',obrigatorio=False)
        self.chNFe = TagCaracter(nome='chNFe', codigo='IR13', tamanho=[44, 44, 44], raiz='//ret/resNFe',obrigatorio=False)
        self.CNPJ = TagCaracter(nome='CNPJ', codigo='IR14', tamanho=[ 0, 14], raiz='//ret/resNFe',obrigatorio=False)
        self.CPF = TagCaracter(nome='CPF', codigo='IR15', tamanho=[11, 11], raiz='//ret/resNFe',obrigatorio=False)
        self.xNome = TagCaracter(nome='xNome', codigo='IR16', tamanho=[ 3, 60], raiz='//ret/resNFe',obrigatorio=False)
        self.IE = TagCaracter(nome='IE', codigo='IR17', tamanho=[ 1, 14], raiz='//ret/resNFe',obrigatorio=False)
        self.dEmi = TagData(nome='dEmi', codigo='IR18', raiz='//ret/resNFe',obrigatorio=False)
        self.tpNF = TagInteiro(nome='tpNF', codigo='IR19', tamanho=[1, 1, 1], raiz='//ret/resNFe',obrigatorio=False)
        self.vNF = TagDecimal(nome='vNF', codigo='IR20', tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='//ret/resNFe',obrigatorio=False)
        self.digVal = TagCaracter(nome='digVal', codigo='IR21', tamanho=[28, 28], raiz='//ret/resNFe',obrigatorio=False)
        self.dhRecbto = TagDataHora(nome='dhRecbto', codigo='IR22', raiz='//ret/resNFe',obrigatorio=False)
        self.cSitNFe = TagInteiro(nome='cSitNFe', codigo='IR23', tamanho=[1, 1, 1], raiz='//ret/resNFe',obrigatorio=False)
        self.cSitConf = TagInteiro(nome='cSitConf', codigo='IR24', tamanho=[1, 1, 1], raiz='//ret/resNFe',obrigatorio=False)


    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        if self.NSU.xml:
            xml += '<ret>'
        xml += self.NSU.xml
        xml += self.chNFe.xml
        xml += self.CNPJ.xml
        xml += self.CPF.xml
        xml += self.xNome.xml
        xml += self.IE.xml
        xml += self.dEmi.xml
        xml += self.tpNF.xml
        xml += self.vNF.xml
        xml += self.digVal.xml
        xml += self.dhRecbto.xml
        xml += self.cSitNFe.xml
        xml += self.cSitConf.xml
        if self.NSU.xml:
            xml += '</resNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NSU.xml = arquivo
            self.chNFe.xml = arquivo
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo
            self.xNome.xml = arquivo
            self.IE.xml = arquivo
            self.dEmi.xml = arquivo
            self.tpNF.xml = arquivo
            self.vNF.xml = arquivo
            self.digVal.xml = arquivo
            self.dhRecbto.xml = arquivo
            self.cSitNFe.xml = arquivo
            self.cSitConf.xml = arquivo




class Ret(XMLNFe):
    def __init__(self):
        super(Ret, self).__init__()
        self.ret = TagCaracter(nome='ret', codigo='IR10', raiz='/')
        self.resNFe = ResNFe()
        self.resCanc = ResCanc()
        self.resCCe = ResCCe()

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += self.resNFe.xml
        xml += self.resCanc.xml
        xml += self.resCCe.xml
        xml += '</ret>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.ret.xml = arquivo
            self.resNFe.xml = arquivo
            self.resCanc.xml = arquivo
            self.resCCe.xml = arquivo




class RetConsNFeDest(XMLNFe):
    def __init__(self):
        super(RetConsNFeDest, self).__init__()
        self.versao = TagDecimal(nome='retConsNFeDest',propriedade='versao', codigo='IR01', namespace=NAMESPACE_NFE, valor='1.01', raiz='/')
        self.tpAmb = TagInteiro(nome='tpAmb', codigo='IR03', tamanho=[ 1, 1, 1], raiz='//retConsNFeDest', valor=2)
        self.verAplic = TagCaracter(nome='verAplic', codigo='IR04', tamanho=[1, 20], raiz='//retConsNFeDest')
        self.cStat = TagCaracter(nome='cStat', codigo='IR05', tamanho=[3, 3, 3], raiz='//retConsNFeDest')
        self.xMotivo = TagCaracter(nome='xMotivo', codigo='IR06', tamanho=[1, 255], raiz='//retConsNFeDest')
        self.dhResp = TagCaracter(nome='dhResp', codigo='IR07', tamanho=[ 0, 30],raiz='//retConsNFeDest')
        self.indCont = TagInteiro(nome='indCont', codigo='IR08', tamanho=[1, 1, 1], raiz='//retConsNFeDest', obrigatorio=False)
        self.ultNSU = TagInteiro(nome='ultNSU', codigo='IR09', tamanho=[1, 15], raiz='//retConsNFeDest',obrigatorio=False)
        self.ret = []
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retconsNFeDest_v1.01.xsd'

    @property
    def xml(self):


        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.dhResp.xml
        xml += self.indCont.xml
        xml += self.ultNSU.xml
        for d in self.ret:
            xml += d.xml

        xml += '</retConsNFeDest>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.dhResp.xml = arquivo
            self.indCont.xml = arquivo
            self.ultNSU.xml = arquivo
            #
            # Técnica para leitura de tags múltiplas
            # As classes dessas tags, e suas filhas, devem ser
            # "reenraizadas" (propriedade raiz) para poderem ser
            # lidas corretamente
            #
            self.ret = self.le_grupo('//retConsNFeDest/ret', Ret)



