# -*- coding: utf-8 -*-

from pynfe.xml_sped.assinatura import Signature
from pynfe.xml_sped.base import *
from ..manual_300 import nfe_110
from . import ESQUEMA_ATUAL
import os

DIRNAME = os.path.dirname(__file__)


class Deduc(XMLNFe):
    def __init__(self):
        super(Deduc, self).__init__()
        self.xDed = TagCaracter(nome='xDed', codigo='ZC11', tamanho=[1, 60], raiz='//deduc')
        self.vDed = TagDecimal(
            nome='vDed', codigo='ZC12', tamanho=[1, 15, 1], decimais=[1, 2, 2], raiz='//deduc')

    def get_xml(self):
        if not (self.xDed.valor or self.vDed.valor):
            return ''
        xml = '<deduc>'
        xml += self.xDed.xml
        xml += self.vDed.xml
        xml += '</deduc>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.xDed.xml = arquivo
            self.vDed.xml = arquivo

    xml = property(get_xml, set_xml)


class ForDia(XMLNFe):
    def __init__(self):
        super(ForDia, self).__init__()
        self.dia = TagInteiro(nome='dia', codigo='ZC05', tamanho=[1, 2, 1], raiz='//forDia')
        self.qtde = TagDecimal(
            nome='qtde', codigo='ZC06', tamanho=[1, 11, 1], decimais=[1, 10, 10], raiz='//forDia')

    def get_xml(self):
        if not (self.dia.valor or self.qtde.valor):
            return ''
        xml = '<forDia>'
        xml += self.dia.xml
        xml += self.qtde.xml
        xml += '</forDia>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.dia.xml = arquivo
            self.qtde.xml = arquivo

    xml = property(get_xml, set_xml)


class Cana(XMLNFe):
    def __init__(self):
        super(Cana, self).__init__()
        self.safra = TagCaracter(
            nome='safra', codigo='ZC02', tamanho=[4, 9], raiz='//NFe/infNFe/cana')
        self.ref = TagCaracter(nome='ref', codigo='ZC03', tamanho=[6, 6], raiz='//NFe/infNFe/cana')
        self.forDia = []
        self.qTotMes = TagDecimal(
            nome='qTotMes', codigo='ZC07', tamanho=[1, 11, 1], decimais=[1, 10, 10],
            raiz='//NFe/infNFe/cana')
        self.qTotAnt = TagDecimal(
            nome='qTotAnt', codigo='ZC08', tamanho=[1, 11, 1], decimais=[1, 10, 10],
            raiz='//NFe/infNFe/cana')
        self.qTotGer = TagDecimal(
            nome='qTotGer', codigo='ZC09', tamanho=[1, 11, 1], decimais=[1, 10, 10],
            raiz='//NFe/infNFe/cana')
        self.deduc = []
        self.vFor = TagDecimal(
            nome='vFor', codigo='ZC13', tamanho=[1, 15, 1], decimais=[1, 2, 2],
            raiz='//NFe/infNFe/cana')
        self.vTotDed = TagDecimal(
            nome='vTotDed', codigo='ZC14', tamanho=[1, 15, 1], decimais=[1, 2, 2],
            raiz='//NFe/infNFe/cana')
        self.vLiqFor = TagDecimal(
            nome='vLiqFor', codigo='ZC15', tamanho=[1, 15, 1], decimais=[1, 2, 2],
            raiz='//NFe/infNFe/cana')

    def get_xml(self):
        if not self.safra.valor:
            return ''
        xml = '<cana>'
        xml += self.safra.xml
        xml += self.ref.xml
        for fd in self.forDia:
            xml += fd.xml
        xml += self.qTotMes.xml
        xml += self.qTotAnt.xml
        xml += self.qTotGer.xml
        for d in self.deduc:
            xml += d.xml
        xml += self.vFor.xml
        xml += self.vTotDed.xml
        xml += self.vLiqFor.xml
        xml += '</cana>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.safra.xml = arquivo
            self.ref.xml = arquivo
            self.forDia = self.le_grupo('//NFe/infNFe/cana/forDia', ForDia)
            self.qTotMes.xml = arquivo
            self.qTotAnt.xml = arquivo
            self.qTotGer.xml = arquivo
            self.deduc = self.le_grupo('//NFe/infNFe/cana/deduc', Deduc)
            self.vFor.xml = arquivo
            self.vTotDed.xml = arquivo
            self.vLiqFor.xml = arquivo

    xml = property(get_xml, set_xml)


class ISSQN(nfe_110.ISSQN):
    def __init__(self):
        super(ISSQN, self).__init__()
        self.cSitTrib = TagCaracter(
            nome='cSitTrib', codigo='U07', tamanho=[1, 1], raiz='//det/imposto/ISSQN')

    def get_xml(self):
        if not self.cSitTrib.valor:
            return ''
        xml = '<ISSQN>'
        xml += self.vBC.xml
        xml += self.vAliq.xml
        xml += self.vISSQN.xml
        xml += self.cMunFG.xml
        xml += self.cListServ.xml
        xml += self.cSitTrib.xml
        xml += '</ISSQN>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.vBC.xml = arquivo
            self.vAliq.xml = arquivo
            self.vISSQN.xml = arquivo
            self.cMunFG.xml = arquivo
            self.cListServ.xml = arquivo
            self.cSitTrib.xml = arquivo

    xml = property(get_xml, set_xml)


class COFINSST(nfe_110.COFINSST):
    def __init__(self):
        super(COFINSST, self).__init__()


class TagCSTCOFINS(nfe_110.TagCSTCOFINS):
    def __init__(self, *args, **kwargs):
        super(TagCSTCOFINS, self).__init__(*args, **kwargs)


class COFINS(nfe_110.COFINS):
    def __init__(self):
        super(COFINS, self).__init__()


class PISST(nfe_110.PISST):
    def __init__(self):
        super(PISST, self).__init__()


class TagCSTPIS(nfe_110.TagCSTPIS):
    def __init__(self, *args, **kwargs):
        super(TagCSTPIS, self).__init__(*args, **kwargs)


class PIS(nfe_110.PIS):
    def __init__(self):
        super(PIS, self).__init__()


class II(nfe_110.II):
    def __init__(self):
        super(II, self).__init__()


class TagCSTIPI(nfe_110.TagCSTIPI):
    def __init__(self, *args, **kwargs):
        super(TagCSTIPI, self).__init__(*args, **kwargs)


class IPI(nfe_110.IPI):
    def __init__(self):
        super(IPI, self).__init__()


class TagCSOSN(TagCaracter):
    def __init__(self, *args, **kwargs):
        super(TagCSOSN, self).__init__(*args, **kwargs)
        self.nome = 'CSOSN'
        self.codigo = 'N12a'
        self.tamanho = [3, 3]
        self.raiz = ''
        self.grupo_icms = None

    def get_valor(self):
        return self._valor_string

    def set_valor(self, value):
        if self.grupo_icms:
            # Para cada codigo de situacao tributaria, redefinimos a obrigatoriedade das tags do
            # grupo de ICMS
            if value == '101':
                self.grupo_icms.nome_tag = 'ICMSSN101'
                self.grupo_icms.raiz_tag = '//det/imposto/ICMS/ICMSSN101'
                self.grupo_icms.pCredSN.obrigatorio = True
                self.grupo_icms.vCredICMSSN.obrigatorio = True
            elif value in ('102', '103', '300', '400'):
                self.grupo_icms.nome_tag = 'ICMSSN102'
                self.grupo_icms.raiz_tag = '//det/imposto/ICMS/ICMSSN102'
            elif value == '201':
                self.grupo_icms.nome_tag = 'ICMSSN201'
                self.grupo_icms.raiz_tag = '//det/imposto/ICMS/ICMSSN201'
                self.grupo_icms.modBCST.obrigatorio = True
                self.grupo_icms.vBCST.obrigatorio = True
                self.grupo_icms.pICMSST.obrigatorio = True
                self.grupo_icms.vICMSST.obrigatorio = True
                self.grupo_icms.pCredSN.obrigatorio = True
                self.grupo_icms.vCredICMSSN.obrigatorio = True
            elif value in ('202', '203'):
                self.grupo_icms.nome_tag = 'ICMSSN202'
                self.grupo_icms.raiz_tag = '//det/imposto/ICMS/ICMSSN202'
                self.grupo_icms.modBCST.obrigatorio = True
                self.grupo_icms.vBCST.obrigatorio = True
                self.grupo_icms.pICMSST.obrigatorio = True
                self.grupo_icms.vICMSST.obrigatorio = True
            elif value == '500':
                self.grupo_icms.nome_tag = 'ICMSSN500'
                self.grupo_icms.raiz_tag = '//det/imposto/ICMS/ICMSSN500'
                self.grupo_icms.vBCSTRet.obrigatorio = True
                self.grupo_icms.vICMSSTRet.obrigatorio = True
            elif value == '900':
                self.grupo_icms.nome_tag = 'ICMSSN900'
                self.grupo_icms.raiz_tag = '//det/imposto/ICMS/ICMSSN900'
                self.grupo_icms.modBC.obrigatorio = True
                self.grupo_icms.vBC.obrigatorio = True
                self.grupo_icms.pICMS.obrigatorio = True
                self.grupo_icms.vICMS.obrigatorio = True
                self.grupo_icms.modBCST.obrigatorio = True
                self.grupo_icms.vBCST.obrigatorio = True
                self.grupo_icms.pICMSST.obrigatorio = True
                self.grupo_icms.vICMSST.obrigatorio = True
                self.grupo_icms.pCredSN.obrigatorio = True
                self.grupo_icms.vCredICMSSN.obrigatorio = True

            self.grupo_icms.CSOSN._valor_string = value

            # Redefine a raiz para todas as tags do grupo ICMS
            self.grupo_icms.orig.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.CSOSN.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.modBC.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBC.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pRedBC.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pICMS.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMS.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.modBCST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pMVAST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pRedBCST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pICMSST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pCredSN.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vCredICMSSN.raiz = self.grupo_icms.raiz_tag

    valor = property(get_valor, set_valor)


class TagCSTICMS(nfe_110.TagCSTICMS):
    def __init__(self, *args, **kwargs):
        super(TagCSTICMS, self).__init__(*args, **kwargs)

    def get_valor(self):
        return self._valor_string

    def set_valor(self, value):
        super(TagCSTICMS, self).set_valor(value)
        if self.grupo_icms:
            # Definimos todas as tags como nao obrigatorias
            self.grupo_icms.vICMSDeson.obrigatorio = False
            self.grupo_icms.motDesICMS.obrigatorio = False
            self.grupo_icms.vBCSTRet.obrigatorio = False
            self.grupo_icms.vICMSSTRet.obrigatorio = False
            self.grupo_icms.vBCSTDest.obrigatorio = False
            self.grupo_icms.vICMSSTDest.obrigatorio = False
            self.grupo_icms.UFST.obrigatorio = False
            self.grupo_icms.pBCOp.obrigatorio = False

            # Por seguranca, zeramos os valores das tags do grupo ICMS ao redefinirmos o codigo da
            # situacao tributaria
            self.grupo_icms.vICMSDeson.valor = '0.00'
            self.grupo_icms.motDesICMS.valor = 0
            self.grupo_icms.vBCSTRet.valor = '0.00'
            self.grupo_icms.vICMSSTRet.valor = '0.00'
            self.grupo_icms.vBCSTDest.valor = '0.00'
            self.grupo_icms.vICMSSTDest.valor = '0.00'
            self.grupo_icms.UFST.valor = ''
            self.grupo_icms.pBCOp.valor = '0.00'

            # Para cada codigo de situacao tributaria, redefinimos a raiz e a obrigatoriedade das
            # tags do grupo de ICMS
            if value == '10':
                if self.grupo_icms.partilha:
                    self.grupo_icms.nome_tag = 'ICMSPart'
                    self.grupo_icms.raiz_tag = '//det/imposto/ICMS/ICMSPart'
                    self.grupo_icms.pBCOp.obrigatorio = True
                    self.grupo_icms.UFST.obrigatorio = True
            elif value in ('40', '41', '50'):
                if self.grupo_icms.repasse and self.valor == '41':
                    self.grupo_icms.nome_tag = 'ICMSST'
                    self.grupo_icms.raiz_tag = '//det/imposto/ICMS/ICMSST'
                    self.raiz = '//det/imposto/ICMS/ICMSST'
                    self.grupo_icms.vBCSTRet.obrigatorio = True
                    self.grupo_icms.vICMSSTRet.obrigatorio = True
                    self.grupo_icms.vBCSTDest.obrigatorio = True
                    self.grupo_icms.vICMSSTDest.obrigatorio = True
            elif value == '60':
                self.grupo_icms.vBCSTRet.obrigatorio = True
                self.grupo_icms.vICMSSTRet.obrigatorio = True
            elif value == '90':
                if self.grupo_icms.partilha:
                    self.grupo_icms.nome_tag = 'ICMSPart'
                    self.grupo_icms.raiz_tag = '//det/imposto/ICMS/ICMSPart'
                    self.grupo_icms.pBCOp.obrigatorio = True
                    self.grupo_icms.UFST.obrigatorio = True

            # Redefine a raiz para todas as tags do grupo ICMS
            self.grupo_icms.motDesICMS.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSDeson.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCSTDest.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSSTDest.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.UFST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pBCOp.raiz = self.grupo_icms.raiz_tag

    valor = property(get_valor, set_valor)


class ICMS(nfe_110.ICMS):
    def __init__(self):
        super(ICMS, self).__init__()
        self.regime_tributario = False  # Simples Nacional/Tradicional
        self.partilha = False  # Para o grupo ICMSPart
        self.repasse = False  # Para o grupo ICMSST
        self.UFST = TagCaracter(nome='UFST', codigo='N24', tamanho=[2, 2], raiz='')
        self.pBCOp = TagDecimal(
            nome='pBCOp', codigo='N25', tamanho=[1, 5, 1], decimais=[0, 2, 2], raiz='')
        self.vBCSTRet = TagDecimal(
            nome='vBCSTRet', codigo='N26', tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='')
        self.vICMSSTRet = TagDecimal(
            nome='vICMSSTRet', codigo='N27', tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='')
        self.motDesICMS = TagInteiro(nome='motDesICMS', codigo='N28', tamanho=[1, 1], raiz='')
        self.pCredSN = TagDecimal(
            nome='pCredSN', codigo='N29', tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='')
        self.vCredICMSSN = TagDecimal(
            nome='vCredICMSSN', codigo='N30', tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='')
        self.vBCSTDest = TagDecimal(
            nome='vBCSTDest', codigo='N31', tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='')
        self.vICMSSTDest = TagDecimal(
            nome='vICMSSTDest', codigo='N32', tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='')
        self.vICMSDeson = TagDecimal(
            nome='vICMSDeson', codigo='N28a', tamanho=[1, 15, 1], decimais=[0, 2, 2], raiz='')
        self.CST = TagCSTICMS()
        self.CST.grupo_icms = self
        self.CSOSN = TagCSOSN()
        self.CSOSN.grupo_icms = self

    def get_xml(self):
        # Define as tags baseado no codigo da situacao tributaria
        xml = '<ICMS><' + self.nome_tag + '>'
        xml += self.orig.xml
        if self.regime_tributario != 1:  # Se for regime tradicional (nao Simples Nacional)
            xml += self.CST.xml
            if self.CST.valor == '00':
                xml += self.modBC.xml
                xml += self.vBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
            elif self.CST.valor == '10':
                if not self.partilha:
                    xml += self.modBC.xml
                    xml += self.vBC.xml
                    xml += self.pICMS.xml
                    xml += self.vICMS.xml
                    xml += self.modBCST.xml
                    if self.modBCST.valor == 4:  # Somente quando for margem de valor agregado
                        xml += self.pMVAST.xml
                    xml += self.pRedBCST.xml
                    xml += self.vBCST.xml
                    xml += self.pICMSST.xml
                    xml += self.vICMSST.xml
                else:
                    xml += self.modBC.xml
                    xml += self.vBC.xml
                    xml += self.pRedBC.xml
                    xml += self.pICMS.xml
                    xml += self.vICMS.xml
                    xml += self.modBCST.xml
                    if self.modBCST.valor == 4:
                        xml += self.pMVAST.xml  # Somente quando for margem de valor agregado
                    xml += self.pRedBCST.xml
                    xml += self.vBCST.xml
                    xml += self.pICMSST.xml
                    xml += self.vICMSST.xml
                    xml += self.pBCOp.xml
                    xml += self.UFST.xml
            elif self.CST.valor == '20':
                xml += self.modBC.xml
                xml += self.pRedBC.xml
                xml += self.vBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
            elif self.CST.valor == '30':
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:
                    xml += self.pMVAST.xml  # Somente quando for margem de valor agregado
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
            elif self.CST.valor in ('40', '41', '50'):
                if self.repasse and self.CST.valor == '41':
                    xml += self.vBCSTRet.xml
                    xml += self.vICMSSTRet.xml
                    xml += self.vBCSTDest.xml
                    xml += self.vICMSSTDest.xml
                elif self.motDesICMS.valor:
                    xml += self.vICMS.xml
                    xml += self.motDesICMS.xml
            elif self.CST.valor == '51':
                xml += self.modBC.xml
                xml += self.pRedBC.xml
                xml += self.vBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
            elif self.CST.valor == '60':
                xml += self.vBCSTRet.xml
                xml += self.vICMSSTRet.xml
            elif self.CST.valor == '70':
                xml += self.modBC.xml
                xml += self.pRedBC.xml
                xml += self.vBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:
                    xml += self.pMVAST.xml  # Somente quando for margem de valor agregado
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
            elif self.CST.valor == '90':
                xml += self.modBC.xml
                xml += self.vBC.xml
                xml += self.pRedBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:
                    xml += self.pMVAST.xml  # Somente quando for margem de valor agregado
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
                if self.partilha:
                    xml += self.pBCOp.xml
                    xml += self.UFST.xml
        else:  # O regime tributario eh o Simples Nacional
            xml += self.CSOSN.xml
            if self.CSOSN.valor == '101':
                xml += self.pCredSN.xml
                xml += self.vCredICMSSN.xml
            elif self.CSOSN.valor in ('102', '103', '300', '400'):
                pass
            elif self.CSOSN.valor == '201':
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:  # Somente quando for margem de valor agregado
                    xml += self.pMVAST.xml
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
                xml += self.pCredSN.xml
                xml += self.vCredICMSSN.xml
            elif self.CSOSN.valor in ('202', '203'):
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:  # Somente quando for margem de valor agregado
                    xml += self.pMVAST.xml
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
            elif self.CSOSN.valor == '500':
                xml += self.vBCSTRet.xml
                xml += self.vICMSSTRet.xml
            elif self.CSOSN.valor == '900':
                xml += self.modBC.xml
                xml += self.vBC.xml
                xml += self.pRedBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:  # Somente quando for margem de valor agregado
                    xml += self.pMVAST.xml
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
                xml += self.pCredSN.xml
                xml += self.vCredICMSSN.xml
        xml += '</' + self.nome_tag + '></ICMS>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            super(ICMS, self).set_xml(arquivo)
            # Para ler corretamente o ICMS, primeiro temos que descobrir em qual grupo de situacao
            # tributaria ele esta
            self.partilha = False
            self.repasse = False
            if self.CST.valor in ['00', '10', '20', '30', '40', '51', '60', '61', '70', '90']:
                self.regime_tributario = 3
            elif self._le_noh('//det/imposto/ICMS/ICMSPart') is not None:
                self.regime_tributario = 3
                self.partilha = True
                self.CST.valor = '10'
            elif self._le_noh('//det/imposto/ICMS/ICMSST') is not None:
                self.regime_tributario = 3
                self.repasse = True
                self.CST.valor = '41'
            elif self._le_noh('//det/imposto/ICMS/ICMSSN101') is not None:
                self.regime_tributario = 1
                self.CSOSN.valor = '101'
            elif self._le_noh('//det/imposto/ICMS/ICMSSN102') is not None:
                self.regime_tributario = 1
                self.CSOSN.valor = '102'
            elif self._le_noh('//det/imposto/ICMS/ICMSSN201') is not None:
                self.regime_tributario = 1
                self.CSOSN.valor = '201'
            elif self._le_noh('//det/imposto/ICMS/ICMSSN202') is not None:
                self.regime_tributario = 1
                self.CSOSN.valor = '202'
            elif self._le_noh('//det/imposto/ICMS/ICMSSN500') is not None:
                self.regime_tributario = 1
                self.CSOSN.valor = '500'
            elif self._le_noh('//det/imposto/ICMS/ICMSSN900') is not None:
                self.regime_tributario = 1
                self.CSOSN.valor = '900'

            # Agora podemos ler os valores tranquilamente...
            self.orig.xml = arquivo
            self.CST.xml = arquivo
            self.modBC.xml = arquivo
            self.vBC.xml = arquivo
            self.pRedBC.xml = arquivo
            self.pICMS.xml = arquivo
            self.vICMS.xml = arquivo
            self.modBCST.xml = arquivo
            self.pMVAST.xml = arquivo
            self.pRedBCST.xml = arquivo
            self.vBCST.xml = arquivo
            self.pICMSST.xml = arquivo
            self.vICMSST.xml = arquivo
            self.vBCSTRet.xml = arquivo
            self.vICMSSTRet.xml = arquivo
            if self.regime_tributario == 1:
                self.CSOSN.xml = arquivo
                self.pCredSN.xml = arquivo
                self.vCredICMSSN.xml = arquivo
            else:
                self.UFST.xml = arquivo
                self.pBCOp.xml = arquivo
                self.motDesICMS.xml = arquivo
                self.vBCSTDest.xml = arquivo
                self.vICMSSTDest.xml = arquivo

    xml = property(get_xml, set_xml)


class Imposto(nfe_110.Imposto):
    def __init__(self):
        super(Imposto, self).__init__()
        self.vTotTrib = TagDecimal(nome='vTotTrib', codigo='M02', tamanho=[1, 15, 1],
                                   decimais=[0, 2, 2], raiz='//det/imposto', obrigatorio=False)
        self.ICMS = ICMS()
        self.ISSQN = ISSQN()

    def get_xml(self):
        xml = '<imposto>'
        xml += self.get_corpo_xml()
        xml += '</imposto>'
        return xml

    def get_corpo_xml(self):
        xml = self.vTotTrib.xml
        if not self.ISSQN.cSitTrib.valor:  # Enviar ICMS, IPI e II somente quando não for servico
            xml += self.ICMS.xml
            xml += self.IPI.xml
            xml += self.II.xml
        else:
            xml += self.ISSQN.xml
        xml += self.PIS.xml
        xml += self.PISST.xml
        xml += self.COFINS.xml
        xml += self.COFINSST.xml
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            super(Imposto, self).set_xml(arquivo)
            self.vTotTrib.xml = arquivo
            self.ICMS.xml = arquivo
            self.IPI.xml = arquivo
            self.II.xml = arquivo
            self.PIS.xml = arquivo
            self.PISST.xml = arquivo
            self.COFINS.xml = arquivo
            self.COFINSST.xml = arquivo
            self.ISSQN.xml = arquivo

    xml = property(get_xml, set_xml)


class CIDE(nfe_110.CIDE):
    def __init__(self):
        super(CIDE, self).__init__()


class Comb(nfe_110.Comb):
    def get_xml(self):
        if not self.cProdANP.valor:
            return ''
        xml = '<comb>'
        xml += self.cProdANP.xml
        xml += self.CODIF.xml
        xml += self.qTemp.xml
        xml += self.UFCons.xml
        xml += self.CIDE.xml
        xml += '</comb>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cProdANP.xml = arquivo
            self.CODIF.xml = arquivo
            self.qTemp.xml = arquivo
            self.UFCons.xml = arquivo
            self.CIDE.xml = arquivo

    xml = property(get_xml, set_xml)


class Arma(nfe_110.Arma):
    def __init__(self):
        super(Arma, self).__init__()


class Med(nfe_110.Med):
    def __init__(self):
        super(Med, self).__init__()


class VeicProd(nfe_110.VeicProd):
    def __init__(self):
        super(VeicProd, self).__init__()
        self.cilin = TagCaracter(
            nome='cilin', codigo='J07', tamanho=[1, 4], raiz='//det/prod/veicProd')
        self.tpComb = TagCaracter(
            nome='tpComb', codigo='J11', tamanho=[2, 2], raiz='//det/prod/veicProd')
        self.CMT = TagCaracter(
            nome='CMT', codigo='J13', tamanho=[1, 9], raiz='//det/prod/veicProd')
        self.cCorDENATRAN = TagCaracter(
            nome='cCorDENATRAN', codigo='J24', tamanho=[2, 2], raiz='//det/prod/veicProd')
        self.lota = TagInteiro(
            nome='lota', codigo='J25', tamanho=[1, 3], raiz='//det/prod/veicProd')
        self.tpRest = TagInteiro(
            nome='tpRest', codigo='J26', tamanho=[1, 3], raiz='//det/prod/veicProd')

    def get_xml(self):
        if not self.chassi.valor:
            return ''
        xml = '<veicProd>'
        xml += self.tpOp.xml
        xml += self.chassi.xml
        xml += self.cCor.xml
        xml += self.xCor.xml
        xml += self.pot.xml
        xml += self.cilin.xml
        xml += self.pesoL.xml
        xml += self.pesoB.xml
        xml += self.nSerie.xml
        xml += self.tpComb.xml
        xml += self.nMotor.xml
        xml += self.CMT.xml
        xml += self.dist.xml
        xml += self.anoMod.xml
        xml += self.anoFab.xml
        xml += self.tpPint.xml
        xml += self.tpVeic.xml
        xml += self.espVeic.xml
        xml += self.VIN.xml
        xml += self.condVeic.xml
        xml += self.cMod.xml
        xml += self.cCorDENATRAN.xml
        xml += self.lota.xml
        xml += self.tpRest.xml
        xml += '</veicProd>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tpOp.xml = arquivo
            self.chassi.xml = arquivo
            self.cCor.xml = arquivo
            self.xCor.xml = arquivo
            self.pot.xml = arquivo
            self.cilin.xml = arquivo
            self.pesoL.xml = arquivo
            self.pesoB.xml = arquivo
            self.nSerie.xml = arquivo
            self.tpComb.xml = arquivo
            self.nMotor.xml = arquivo
            self.CMT.xml = arquivo
            self.dist.xml = arquivo
            self.anoMod.xml = arquivo
            self.anoFab.xml = arquivo
            self.tpPint.xml = arquivo
            self.tpVeic.xml = arquivo
            self.espVeic.xml = arquivo
            self.VIN.xml = arquivo
            self.condVeic.xml = arquivo
            self.cMod.xml = arquivo
            self.cCorDENATRAN.xml = arquivo
            self.lota.xml = arquivo
            self.tpRest.xml = arquivo

    xml = property(get_xml, set_xml)


class Adi(nfe_110.Adi):
    def __init__(self):
        super(Adi, self).__init__()


class DI(nfe_110.DI):
    def __init__(self):
        super(DI, self).__init__()


class Prod(nfe_110.Prod):
    def __init__(self):
        super(Prod, self).__init__()
        self.NCM = TagCaracter(nome='NCM', codigo='I05', tamanho=[2, 8], raiz='//det/prod')
        self.CEST = TagCaracter(
            nome='CEST', codigo='I05c', tamanho=[1, 7], raiz='//det/prod', obrigatorio=False)
        self.qCom = TagDecimal(
            nome='qCom', codigo='I10', tamanho=[1, 15, 1], decimais=[0, 4, 3], raiz='//det/prod')
        self.vUnCom = TagDecimal(
            nome='vUnCom', codigo='I10a', tamanho=[1, 21, 1], decimais=[0, 10, 3],
            raiz='//det/prod')
        self.qTrib = TagDecimal(
            nome='qTrib', codigo='I14', tamanho=[1, 15, 1], decimais=[0, 4, 4], raiz='//det/prod')
        self.vUnTrib = TagDecimal(
            nome='vUnTrib', codigo='I14a', tamanho=[1, 21, 1], decimais=[0, 10, 4],
            raiz='//det/prod')
        self.vOutro = TagDecimal(
            nome='vOutro', codigo='I17a', tamanho=[1, 15, 1], decimais=[0, 2, 2],
            raiz='//det/prod', obrigatorio=False)
        self.indTot = TagInteiro(
            nome='indTot', codigo='I17b', tamanho=[1, 1, 1], raiz='//det/prod', valor=1)
        self.xPed = TagCaracter(
            nome='xPed', codigo='I30', tamanho=[1, 15], raiz='//det/prod', obrigatorio=False)
        self.nItemPed = TagCaracter(
            nome='nItemPed', codigo='I31', tamanho=[1, 6], raiz='//det/prod', obrigatorio=False)
        self.nFCI = TagCaracter(
            nome='nFCI', codigo='I70', tamanho=[1, 36], raiz='//det/prod', obrigatorio=False)
        self.veicProd = VeicProd()
        self.comb = Comb()

    def get_xml(self):
        xml = '<prod>'
        xml += self.cProd.xml
        xml += self.cEAN.xml
        xml += self.xProd.xml
        xml += self.NCM.xml
        xml += self.CEST.xml
        xml += self.EXTIPI.xml
        xml += self.CFOP.xml
        xml += self.uCom.xml
        xml += self.qCom.xml
        xml += self.vUnCom.xml
        xml += self.vProd.xml
        xml += self.cEANTrib.xml
        xml += self.uTrib.xml
        xml += self.qTrib.xml
        xml += self.vUnTrib.xml
        xml += self.vFrete.xml
        xml += self.vSeg.xml
        xml += self.vDesc.xml
        xml += self.vOutro.xml
        xml += self.indTot.xml
        for d in self.DI:
            xml += d.xml
        xml += self.xPed.xml
        xml += self.nItemPed.xml
        xml += self.nFCI.xml
        xml += self.veicProd.xml
        xml += self.med.xml
        for a in self.arma:
            xml += a.xml
        xml += self.comb.xml
        xml += '</prod>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cProd.xml = arquivo
            self.cEAN.xml = arquivo
            self.xProd.xml = arquivo
            self.NCM.xml = arquivo
            self.CEST.xml = arquivo
            self.EXTIPI.xml = arquivo
            self.genero.xml = arquivo
            self.CFOP.xml = arquivo
            self.uCom.xml = arquivo
            self.qCom.xml = arquivo
            self.vUnCom.xml = arquivo
            self.vProd.xml = arquivo
            self.cEANTrib.xml = arquivo
            self.uTrib.xml = arquivo
            self.qTrib.xml = arquivo
            self.vUnTrib.xml = arquivo
            self.vFrete.xml = arquivo
            self.vSeg.xml = arquivo
            self.vDesc.xml = arquivo
            self.vOutro.xml = arquivo

            #
            # Técnica para leitura de tags múltiplas
            # As classes dessas tags, e suas filhas, devem ser
            # "reenraizadas" (propriedade raiz) para poderem ser
            # lidas corretamente
            #
            self.DI = self.le_grupo('//det/prod/DI', DI)
            self.nFCI.xml = arquivo
            self.veicProd.xml = arquivo

            #
            # Técnica para leitura de tags múltiplas
            # As classes dessas tags, e suas filhas, devem ser
            # "reenraizadas" (propriedade raiz) para poderem ser
            # lidas corretamente
            #
            self.med = self.le_grupo('//det/prod/med', Med)
            self.arma = self.le_grupo('//det/prod/arma', Arma)

            self.comb.xml = arquivo

    xml = property(get_xml, set_xml)


class Det(nfe_110.Det):
    def __init__(self):
        super(Det, self).__init__()
        self.prod = Prod()
        self.imposto = Imposto()

    def cst_formatado(self):
        super(Det, self).cst_formatado()
        formatado = str(self.imposto.ICMS.orig.valor).zfill(1)
        formatado += str(self.imposto.ICMS.CST.valor).zfill(2)
        return formatado


class Compra(nfe_110.Compra):
    def __init__(self):
        super(Compra, self).__init__()


class Exporta(nfe_110.Exporta):
    def __init__(self):
        super(Exporta, self).__init__()


class ProcRef(nfe_110.ProcRef):
    def __init__(self):
        super(ProcRef, self).__init__()


class ObsFisco(nfe_110.ObsFisco):
    def __init__(self):
        super(ObsFisco, self).__init__()


class ObsCont(nfe_110.ObsCont):
    def __init__(self):
        super(ObsCont, self).__init__()


class InfAdic(nfe_110.InfAdic):
    def __init__(self):
        super(InfAdic, self).__init__()
        self.infAdFisco = TagCaracter(
            nome='infAdFisco', codigo='Z02', tamanho=[1, 2000], raiz='//NFe/infNFe/infAdic',
            obrigatorio=False)


class Dup(nfe_110.Dup):
    def __init__(self):
        super(Dup, self).__init__()


class Fat(nfe_110.Fat):
    def __init__(self):
        super(Fat, self).__init__()


class Cobr(nfe_110.Cobr):
    def __init__(self):
        super(Cobr, self).__init__()


class Lacres(nfe_110.Lacres):
    def __init__(self):
        super(Lacres, self).__init__()


class Vol(nfe_110.Vol):
    def __init__(self):
        super(Vol, self).__init__()


class Reboque(nfe_110.Reboque):
    def __init__(self):
        super(Reboque, self).__init__()


class VeicTransp(nfe_110.VeicTransp):
    def __init__(self):
        super(VeicTransp, self).__init__()


class RetTransp(nfe_110.RetTransp):
    def __init__(self):
        super(RetTransp, self).__init__()


class Transporta(nfe_110.Transporta):
    def __init__(self):
        super(Transporta, self).__init__()


class Transp(nfe_110.Transp):
    def __init__(self):
        super(Transp, self).__init__()
        self.vagao = TagCaracter(
            nome='vagao', codigo='X25a', tamanho=[1, 20], raiz='//NFe/infNFe/transp',
            obrigatorio=False)
        self.balsa = TagCaracter(
            nome='balsa', codigo='X25b', tamanho=[1, 20], raiz='//NFe/infNFe/transp',
            obrigatorio=False)

    def get_xml(self):
        xml = '<transp>'
        xml += self.modFrete.xml
        xml += self.transporta.xml
        xml += self.retTransp.xml
        if self.balsa.valor:
            xml += self.balsa.xml
        elif self.vagao.valor:
            xml += self.vagao.xml
        else:
            xml += self.veicTransp.xml
            for r in self.reboque:
                xml += r.xml
        for v in self.vol:
            xml += v.xml
        xml += '</transp>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.modFrete.xml = arquivo
            self.transporta.xml = arquivo
            self.retTransp.xml = arquivo
            self.veicTransp.xml = arquivo

            #
            # Técnica para leitura de tags múltiplas
            # As classes dessas tags, e suas filhas, devem ser
            # "reenraizadas" (propriedade raiz) para poderem ser
            # lidas corretamente
            #
            self.reboque = self.le_grupo('//NFe/infNFe/transp/reboque', nfe_110.Reboque)

            self.vagao.xml = arquivo
            self.balsa.xml = arquivo

            self.vol = self.le_grupo('//NFe/infNFe/transp/vol', nfe_110.Vol)

    xml = property(get_xml, set_xml)


class RetTrib(nfe_110.RetTrib):
    def __init__(self):
        super(RetTrib, self).__init__()


class ISSQNTot(nfe_110.ISSQNTot):
    def __init__(self):
        super(ISSQNTot, self).__init__()


class ICMSTot(nfe_110.ICMSTot):
    def __init__(self):
        super(ICMSTot, self).__init__()


class Total(nfe_110.Total):
    def __init__(self):
        super(Total, self).__init__()


class Entrega(nfe_110.Entrega):
    def __init__(self):
        super(Entrega, self).__init__()
        self.CNPJ = TagCaracter(
            nome='CNPJ', codigo='G02', tamanho=[0, 14], raiz='//NFe/infNFe/retirada')
        self.CPF = TagCaracter(
            nome='CPF', codigo='G02a', tamanho=[11, 11], raiz='//NFe/infNFe/retirada')

    def get_xml(self):
        if not (self.CNPJ.valor or self.CPF.valor):
            return ''
        xml = '<entrega>'
        if self.CPF.valor:
            xml += self.CPF.xml
        else:
            xml += self.CNPJ.xml
        xml += self.xLgr.xml
        xml += self.nro.xml
        xml += self.xCpl.xml
        xml += self.xBairro.xml
        xml += self.cMun.xml
        xml += self.xMun.xml
        xml += self.UF.xml
        xml += '</entrega>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo
            self.xLgr.xml = arquivo
            self.nro.xml = arquivo
            self.xCpl.xml = arquivo
            self.xBairro.xml = arquivo
            self.cMun.xml = arquivo
            self.xMun.xml = arquivo
            self.UF.xml = arquivo

    xml = property(get_xml, set_xml)


class Retirada(nfe_110.Retirada):
    def __init__(self):
        super(Retirada, self).__init__()
        self.CNPJ = TagCaracter(
            nome='CNPJ', codigo='F02', tamanho=[0, 14], raiz='//NFe/infNFe/retirada')
        self.CPF = TagCaracter(
            nome='CPF', codigo='F02a', tamanho=[11, 11], raiz='//NFe/infNFe/retirada')

    def get_xml(self):
        if not (self.CNPJ.valor or self.CPF.valor):
            return ''
        xml = '<retirada>'
        if self.CPF.valor:
            xml += self.CPF.xml
        else:
            xml += self.CNPJ.xml
        xml += self.xLgr.xml
        xml += self.nro.xml
        xml += self.xCpl.xml
        xml += self.xBairro.xml
        xml += self.cMun.xml
        xml += self.xMun.xml
        xml += self.UF.xml
        xml += '</retirada>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo
            self.xLgr.xml = arquivo
            self.nro.xml = arquivo
            self.xCpl.xml = arquivo
            self.xBairro.xml = arquivo
            self.cMun.xml = arquivo
            self.xMun.xml = arquivo
            self.UF.xml = arquivo

    xml = property(get_xml, set_xml)


class EnderDest(nfe_110.EnderDest):
    def __init__(self):
        super(EnderDest, self).__init__()
        self.fone = TagInteiro(
            nome='fone', codigo='E16', tamanho=[6, 14], raiz='//NFe/infNFe/dest/enderDest',
            obrigatorio=False)


class Dest(nfe_110.Dest):
    def __init__(self):
        super(Dest, self).__init__()
        self.enderDest = EnderDest()
        self.ISUF = TagCaracter(
            nome='ISUF', codigo='E18', tamanho=[8, 9], raiz='//NFe/infNFe/dest',
            obrigatorio=False)
        self.email = TagCaracter(
            nome='email', codigo='E19', tamanho=[1, 60], raiz='//NFe/infNFe/dest',
            obrigatorio=False)

    def get_xml(self):
        xml = '<dest>'
        if self.CPF.valor:
            xml += self.CPF.xml
        else:
            xml += self.CNPJ.xml
        xml += self.xNome.xml
        xml += self.enderDest.xml
        xml += self.IE.xml
        xml += self.ISUF.xml
        xml += self.email.xml
        xml += '</dest>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo
            self.xNome.xml = arquivo
            self.enderDest.xml = arquivo
            self.IE.xml = arquivo
            self.ISUF.xml = arquivo
            self.email.xml = arquivo

    xml = property(get_xml, set_xml)


class Avulsa(nfe_110.Avulsa):
    def __init__(self):
        super(Avulsa, self).__init__()
        self.fone = TagInteiro(
            nome='fone', codigo='D05', tamanho=[6, 14], raiz='//NFe/infNFe/avulsa')


class EnderEmit(nfe_110.EnderEmit):
    def __init__(self):
        super(EnderEmit, self).__init__()
        self.fone = TagInteiro(
            nome='fone', codigo='C16', tamanho=[6, 14], raiz='//NFe/infNFe/emit/enderEmit',
            obrigatorio=False)


class Emit(nfe_110.Emit):
    def __init__(self):
        super(Emit, self).__init__()
        self.enderEmit = EnderEmit()
        self.CRT = TagInteiro(
            nome='CRT', codigo='C21', tamanho=[1, 1], raiz='//NFe/infNFe/emit', valor=1)

    def get_xml(self):
        xml = '<emit>'
        xml += self.CNPJ.xml
        xml += self.CPF.xml
        xml += self.xNome.xml
        xml += self.xFant.xml
        xml += self.enderEmit.xml
        xml += self.IE.xml
        xml += self.IEST.xml
        xml += self.IM.xml
        xml += self.CNAE.xml
        xml += self.CRT.xml
        xml += '</emit>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo
            self.xNome.xml = arquivo
            self.xFant.xml = arquivo
            self.enderEmit.xml = arquivo
            self.IE.xml = arquivo
            self.IEST.xml = arquivo
            self.IM.xml = arquivo
            self.CNAE.xml = arquivo
            self.CRT.xml = arquivo

    xml = property(get_xml, set_xml)


class RefECF(XMLNFe):
    def __init__(self):
        super(RefECF, self).__init__()
        self.mod = TagCaracter(nome='mod', codigo='B20l', tamanho=[2, 2, 2], raiz='//NFref/refECF')
        self.nECF = TagInteiro(nome='nECF', codigo='B20m', tamanho=[1, 3, 1], raiz='//NFref/refECF')
        self.nCOO = TagInteiro(nome='nCOO', codigo='B20n', tamanho=[1, 6, 1], raiz='//NFref/refECF')

    def get_xml(self):
        if not (self.mod.valor or self.nECF.valor or self.nCOO.valor):
            return ''
        xml = '<refECF>'
        xml += self.mod.xml
        xml += self.nECF.xml
        xml += self.nCOO.xml
        xml += '</refECF>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.mod.xml = arquivo
            self.nECF.xml = arquivo
            self.nCOO.xml = arquivo

    xml = property(get_xml, set_xml)


class RefNFP(XMLNFe):
    def __init__(self):
        super(RefNFP, self).__init__()
        self.cUF = TagInteiro(nome='cUF', codigo='B20b', tamanho=[2, 2, 2], raiz='//NFref/refNFP')
        self.AAMM = TagCaracter(
            nome='AAMM', codigo='B20c', tamanho=[4, 4, 4], raiz='//NFref/refNFP')
        self.CNPJ = TagCaracter(nome='CNPJ', codigo='B20d', tamanho=[14, 14], raiz='//NFref/refNFP')
        self.CPF = TagCaracter(nome='CPF', codigo='B20e', tamanho=[11, 11], raiz='//NFref/refNFP')
        self.IE = TagCaracter(nome='IE', codigo='B20f', tamanho=[1, 14], raiz='//NFref/refNFP')
        self.mod = TagCaracter(nome='mod', codigo='B20g', tamanho=[2, 2, 2], raiz='//NFref/refNFP')
        self.serie = TagInteiro(
            nome='serie', codigo='B20h', tamanho=[1, 3, 1], raiz='//NFref/refNFP')
        self.nNF = TagInteiro(nome='nNF', codigo='B20i', tamanho=[1, 9, 1], raiz='//NFref/refNFP')

    def get_xml(self):
        if not (self.cUF.valor or self.AAMM.valor or self.CNPJ.valor or self.CPF.valor or
                self.IE.valor or self.mod.valor or self.serie.valor or self.nNF.valor):
            return ''
        xml = '<refNFP>'
        xml += self.cUF.xml
        xml += self.AAMM.xml
        if self.CPF.valor:
            xml += self.CPF.xml
        else:
            xml += self.CNPJ.xml
        xml += self.IE.xml
        xml += self.mod.xml
        xml += self.serie.xml
        xml += self.nNF.xml
        xml += '</refNFP>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cUF.xml = arquivo
            self.AAMM.xml = arquivo
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo
            self.IE.xml = arquivo
            self.mod.xml = arquivo
            self.serie.xml = arquivo
            self.nNF.xml = arquivo

    xml = property(get_xml, set_xml)


class RefNF(nfe_110.RefNF):
    def __init__(self):
        super(RefNF, self).__init__()


class NFRef(nfe_110.NFRef):
    def __init__(self):
        super(NFRef, self).__init__()
        self.refNFP = RefNFP()
        self.refCTe = TagCaracter(
            nome='refCTe', codigo='B20j', tamanho=[44, 44], raiz='//NFref', obrigatorio=False)
        self.refECF = RefECF()

    def get_xml(self):
        if not (self.refNFe.valor or self.refNF.xml or self.refNFP.xml or self.refCTe.valor or
                self.refECF.xml):
            return ''
        xml = '<NFref>'
        if self.refNFe.valor:
            xml += self.refNFe.xml
        elif self.refNF.xml:
            xml += self.refNF.xml
        elif self.refNFP.xml:
            xml += self.refNFP.xml
        elif self.refCTe.valor:
            xml += self.refCTe.xml
        elif self.refECF.xml:
            xml += self.refECF.xml
        xml += '</NFref>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.refNFe.xml = arquivo
            self.refNF.xml = arquivo
            self.refNFP.xml = arquivo
            self.refCTe.xml = arquivo
            self.refECF.xml = arquivo

    xml = property(get_xml, set_xml)


class Ide(nfe_110.Ide):
    def __init__(self):
        super(Ide, self).__init__()
        self.cNF = TagCaracter(nome='cNF', codigo='B03', tamanho=[8, 8, 8], raiz='//NFe/infNFe/ide')
        self.hSaiEnt = TagHora(
            nome='hSaiEnt', codigo='B10a', raiz='//NFe/infNFe/ide', obrigatorio=False)
        self.dhCont = TagDataHora(
            nome='dhCont', codigo='B28', raiz='//NFe/infNFe/ide', obrigatorio=False)
        self.xJust = TagCaracter(
            nome='xJust', codigo='B29', raiz='//NFe/infNFe/ide', obrigatorio=False)

    def get_xml(self):
        xml = '<ide>'
        xml += self.cUF.xml
        xml += self.cNF.xml
        xml += self.natOp.xml
        xml += self.indPag.xml
        xml += self.mod.xml
        xml += self.serie.xml
        xml += self.nNF.xml
        xml += self.dEmi.xml
        xml += self.dSaiEnt.xml
        xml += self.hSaiEnt.xml
        xml += self.tpNF.xml
        xml += self.cMunFG.xml
        for nr in self.NFref:
            xml += nr.xml
        xml += self.tpImp.xml
        xml += self.tpEmis.xml
        xml += self.cDV.xml
        xml += self.tpAmb.xml
        xml += self.finNFe.xml
        xml += self.procEmi.xml
        xml += self.verProc.xml
        xml += self.dhCont.xml
        xml += self.xJust.xml
        xml += '</ide>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cUF.xml = arquivo
            self.cNF.xml = arquivo
            self.natOp.xml = arquivo
            self.indPag.xml = arquivo
            self.mod.xml = arquivo
            self.serie.xml = arquivo
            self.nNF.xml = arquivo
            self.dEmi.xml = arquivo
            self.dSaiEnt.xml = arquivo
            self.hSaiEnt.xml = arquivo
            self.tpNF.xml = arquivo
            self.cMunFG.xml = arquivo

            #
            # Técnica para leitura de tags múltiplas
            # As classes dessas tags, e suas filhas, devem ser
            # "reenraizadas" (propriedade raiz) para poderem ser
            # lidas corretamente
            #
            self.NFref = self.le_grupo('//NFe/infNFe/ide/NFref', NFRef)

            self.tpImp.xml = arquivo
            self.tpEmis.xml = arquivo
            self.cDV.xml = arquivo
            self.tpAmb.xml = arquivo
            self.finNFe.xml = arquivo
            self.procEmi.xml = arquivo
            self.verProc.xml = arquivo
            self.dhCont.xml = arquivo
            self.xJust.xml = arquivo

    xml = property(get_xml, set_xml)


class InfNFe(nfe_110.InfNFe):
    def __init__(self):
        super(InfNFe, self).__init__()
        self.versao = TagDecimal(
            nome='infNFe', codigo='A01', propriedade='versao', raiz='//NFe',
            namespace=NAMESPACE_NFE, valor='2.00')
        self.ide = Ide()
        self.emit = Emit()
        self.avulsa = Avulsa()
        self.dest = Dest()
        self.retirada = Retirada()
        self.entrega = Entrega()
        self.det = []
        self.total = Total()
        self.transp = Transp()
        self.cobr = Cobr()
        self.infAdic = InfAdic()
        self.exporta = Exporta()
        self.compra = Compra()
        self.cana = Cana()

    def get_xml(self):
        xml = '<infNFe versao="' + str(self.versao.valor) + '" Id="' + self.Id.valor + '">'
        xml += self.ide.xml
        xml += self.emit.xml
        xml += self.avulsa.xml
        xml += self.dest.xml
        xml += self.retirada.xml
        xml += self.entrega.xml
        for d in self.det:
            xml += d.xml
        xml += self.total.xml
        xml += self.transp.xml
        xml += self.cobr.xml
        xml += self.infAdic.xml
        xml += self.exporta.xml
        xml += self.compra.xml
        xml += self.cana.xml
        xml += '</infNFe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.Id.xml = arquivo
            self.ide.xml = arquivo
            self.emit.xml = arquivo
            self.avulsa.xml = arquivo
            self.dest.xml = arquivo
            self.retirada.xml = arquivo
            self.entrega.xml = arquivo

            #
            # Técnica para leitura de tags múltiplas
            # As classes dessas tags, e suas filhas, devem ser
            # "reenraizadas" (propriedade raiz) para poderem ser
            # lidas corretamente
            #
            self.det = self.le_grupo('//NFe/infNFe/det', Det)

            self.total.xml = arquivo
            self.transp.xml = arquivo
            self.cobr.xml = arquivo
            self.infAdic.xml = arquivo
            self.exporta.xml = arquivo
            self.compra.xml = arquivo
            self.cana.xml = arquivo

    xml = property(get_xml, set_xml)


class NFe(nfe_110.NFe):
    def __init__(self):
        super(NFe, self).__init__()
        self.infNFe = InfNFe()
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'nfe_v2.00.xsd'

    def gera_nova_chave(self):

        super(NFe, self).gera_nova_chave()

        #
        # Ajustar o campo cNF para remover o 1º dígito, que é
        # o tipo da emissão
        #
        self.infNFe.ide.cNF.valor = self.chave[35:43]

    def monta_chave(self):
        self.gera_nova_chave()
        chave = str(self.infNFe.ide.cUF.valor).strip().rjust(2, '0')
        chave += str(self.infNFe.ide.dEmi.valor.strftime('%y%m')).strip().rjust(4, '0')
        chave += str(self.infNFe.emit.CNPJ.valor).strip().rjust(14, '0')
        chave += '55'
        chave += str(self.infNFe.ide.serie.valor).strip().rjust(3, '0')
        chave += str(self.infNFe.ide.nNF.valor).strip().rjust(9, '0')

        #
        # Inclui agora o tipo da emissão
        #
        chave += str(self.infNFe.ide.tpEmis.valor).strip().rjust(1, '0')

        chave += str(self.infNFe.ide.cNF.valor).strip().rjust(8, '0')
        chave += str(self.infNFe.ide.cDV.valor).strip().rjust(1, '0')

        self.chave = chave
