# -*- coding: utf-8 -*-

from pynfe.xml_sped.base import TagDecimal, NAMESPACE_NFE, ABERTURA, XMLNFe
from ..manual_500 import ESQUEMA_ATUAL, ProtNFe_310, RetCancNFe_310
from ..manual_401 import conssitnfe_200
import os


DIRNAME = os.path.dirname(__file__)


class ConsSitNFe(conssitnfe_200.ConsSitNFe):
    def __init__(self):
        super(ConsSitNFe, self).__init__()
        self.versao = TagDecimal(
            nome='consSitNFe', codigo='EP01', propriedade='versao', namespace=NAMESPACE_NFE,
            valor='3.10', raiz='/')
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'consSitNFe_v3.10.xsd'


class RetConsSitNFe(conssitnfe_200.RetConsSitNFe):
    def __init__(self):
        super(RetConsSitNFe, self).__init__()
        self.versao = TagDecimal(
            nome='retConsSitNFe', codigo='ER01', propriedade='versao', namespace=NAMESPACE_NFE,
            valor='3.10', raiz='/')
        self.protNFe = None
        self.retCancNFe = None
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retConsSitNFe_v3.10.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml
        xml += self.chNFe.xml
        if self.protNFe is not None:
            xml += self.protNFe.xml
        if self.retCancNFe is not None:
            xml += self.retCancNFe.raw_xml()
        xml += '</retConsSitNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.cUF.xml = arquivo
            self.chNFe.xml = arquivo
            if self._le_noh('//retConsSitNFe/protNFe') is not None:
                self.protNFe = ProtNFe_310()
                self.protNFe.xml = arquivo
            if self._le_noh('//retConsSitNFe/retCancNFe') is not None:
                self.retCancNFe = RetCancNFe_310()
                self.retCancNFe.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
