# -*- encoding: utf-8 -*-
# #############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################


from pynfe.xml_sped.base import *

import os

DIRNAME = os.path.dirname(__file__)


class CodServico(XMLNFe):
    def __init__(self):
        super(CodServico, self).__init__()
        self.codservico = TagCaracter(
            nome='codservico', codigo='', tamanho=[1, 15], raiz='/codservico')

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += '\t\t' + self.codservico.xml + '\n'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.codservico.xml = arquivo



class Importacao(XMLNFe):
    def __init__(self):
        super(Importacao, self).__init__()
        self.rps_numero = TagCaracter(nome='rps_numero', codigo='', tamanho=[0, 20], raiz='//importacao')#, obrigatorio=False)
        self.rps_data = TagData(nome='rps_data', codigo='', raiz='//importacao/nota')#, obrigatorio=False)
        self.tomador_nome = TagCaracter(nome='tomador_nome', codigo='', tamanho=[0, 100], raiz='//importacao/nota')
        self.tomador_cnpjcpf = TagCaracter(nome='tomador_cnpjcpf', codigo='', tamanho=[13, 18], raiz='//importacao/nota')
        self.tomador_inscrmunicipal = TagCaracter(nome='tomador_inscrmunicipal', codigo='', tamanho=[0, 20], raiz='//importacao/nota')#, obrigatorio=False)
        self.tomador_endereco = TagCaracter(nome='tomador_endereco', codigo='', tamanho=[0, 40], raiz='//importacao/nota')#, obrigatorio=False)
        self.tomador_cep = TagCaracter(nome='tomador_cep', codigo='', tamanho=[1, 12], raiz='//importacao/nota')#, obrigatorio=False)
        self.tomador_municipio = TagCaracter(nome='tomador_municipio', codigo='', tamanho=[1, 100], raiz='//importacao/nota')#, obrigatorio=False)
        self.tomador_uf = TagCaracter(nome='tomador_uf', codigo='', tamanho=[0, 3], raiz='//importacao/nota')#, obrigatorio=False)
        self.tomador_email = TagCaracter(nome='tomador_email', codigo='', tamanho=[1, 100], raiz='//importacao/nota')#, obrigatorio=False)
        self.discriminacao = TagCaracter(nome='discriminacao', codigo='', tamanho=[1, 400], raiz='//importacao/nota')#, obrigatorio=False)
        self.valortotal = TagDecimal(nome='valortotal', codigo='', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz='//importacao/nota')#, obrigatorio=False)
        self.valordeducoes = TagDecimal(nome='valordeducoes', codigo='', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz='//importacao/nota')#, obrigatorio=False)
        self.basecalculo = TagDecimal(nome='basecalculo', codigo='', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz='//importacao/nota')#, obrigatorio=False)
        self.valoriss = TagDecimal(nome='valoriss', codigo='', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz='//importacao/nota')#, obrigatorio=False)
        self.issretido = TagDecimal(nome='issretido', codigo='', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz='//importacao/nota')#, obrigatorio=False)
        self.credito = TagDecimal(nome='credito', codigo='', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz='//importacao/nota')#, obrigatorio=False)
        self.estado = TagCaracter(nome='estado', codigo='', tamanho=[1, 20], raiz='//importacao/nota')
        self.codservico = []  # CodServico()

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += '<?xml version="1.0" encoding="iso-8859-1"?>\n'
        xml += '<importacao>\n'
        xml += '\t<nota>\n'
        xml += '\t\t' + self.rps_numero.xml + '\n'
        xml += '\t\t' + self.rps_data.xml + '\n'
        xml += '\t\t' + self.tomador_nome.xml + '\n'
        xml += '\t\t' + self.tomador_cnpjcpf.xml + '\n'
        xml += '\t\t' + self.tomador_inscrmunicipal.xml + '\n'
        xml += '\t\t' + self.tomador_endereco.xml + '\n'
        xml += '\t\t' + self.tomador_cep.xml + '\n'
        xml += '\t\t' + self.tomador_municipio.xml + '\n'
        xml += '\t\t' + self.tomador_uf.xml + '\n'
        xml += '\t\t' + self.tomador_email.xml + '\n'
        xml += '\t\t' + self.discriminacao.xml + '\n'
        xml += '\t\t' + self.valortotal.xml + '\n'
        xml += '\t\t' + self.valordeducoes.xml + '\n'
        xml += '\t\t' + self.basecalculo.xml + '\n'
        xml += '\t\t' + self.valoriss.xml + '\n'
        xml += '\t\t' + self.issretido.xml + '\n'
        xml += '\t\t' + self.credito.xml + '\n'
        xml += '\t\t' + self.estado.xml + '\n'
        for d in self.codservico:
            xml += d.xml
        xml += '\t' + '</nota>' + '\n'
        xml += '</importacao>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.rps_numero.xml = arquivo
            self.rps_data.xml = arquivo
            self.tomador_nome.xml = arquivo
            self.tomador_cnpjcpf.xml = arquivo
            self.tomador_inscrmunicipal.xml = arquivo
            self.tomador_endereco.xml = arquivo
            self.tomador_cep.xml = arquivo
            self.tomador_municipio.xml = arquivo
            self.tomador_uf.xml = arquivo
            self.tomador_email.xml = arquivo
            self.discriminacao.xml = arquivo
            self.valortotal.xml = arquivo
            self.valordeducoes.xml = arquivo
            self.basecalculo.xml = arquivo
            self.valoriss.xml = arquivo
            self.issretido.xml = arquivo
            self.credito.xml = arquivo
            self.estado.xml = arquivo
            self.ret = self.le_grupo('//importacao/nota/codservico', CodServico)


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
