# -*- coding: utf-8 -*-

from pynfe.nfse.base_nfse import *
import os


DIRNAME = os.path.dirname(__file__)

# -------------------- Tipos Complexos -------------------

class Cabecalho(XMLNFSe):
    def __init__(self):
        super(Cabecalho, self).__init__()
        self.nome_tag = 'cabecalho'
        self.Versao = TagInteiro(nome='Versao', codigo='Versao', tamanho=[1,4,1],
                                 namespace=NAMESPACE_NFSE)
        self.versaoDados = TagInteiro(nome='versaoDados', codigo='versaoDados', tamanho=[1,4,1])

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<?xml version="1.0"?>')
        xml.append('<%s Versao="%s">' % (self.nome_tag, self.Versao.xml))
        xml.append(self.versaoDados.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Versao.xml = arquivo
            self.versaoDados.xml = arquivo


class Signature(XMLNFSe):
    def __init__(self):
        super(Signature, self).__init__()
        self.URI = ''
        self.Id = ''
        self.DigestValue = ''
        self.SignatureValue = ''
        self.X509Certificate = ''
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/')
        self.arquivo_esquema = 'xmldsig-core-schema_v1.01.xsd'

    @property
    def xml(self):
        if not len(self.URI):
            self.URI = '#'
        if self.URI[0] != '#':
            self.URI = '#' + self.URI
        xml = '<Signature xmlns="http://www.w3.org/2000/09/xmldsig#" Id="%s">' % self.Id
        xml += '<SignedInfo>'
        xml += '<CanonicalizationMethod Algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315"/>'
        xml += '<SignatureMethod Algorithm="http://www.w3.org/2000/09/xmldsig#rsa-sha1"/>'
        xml += '<Reference URI="' + self.URI + '">'
        xml += '<Transforms>'
        xml += '<Transform Algorithm="http://www.w3.org/2000/09/xmldsig#enveloped-signature"/>'
        xml += '<Transform Algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315"/>'
        xml += '</Transforms>'
        xml += '<DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>'
        xml += '<DigestValue>' + self.DigestValue + '</DigestValue>'
        xml += '</Reference>'
        xml += '</SignedInfo>'
        xml += '<SignatureValue>' + self.SignatureValue + '</SignatureValue>'
        xml += '<KeyInfo>'
        xml += '<X509Data>'
        xml += '<X509Certificate>' + self.X509Certificate + '</X509Certificate>'
        xml += '</X509Data>'
        xml += '</KeyInfo>'
        xml += '</Signature>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id = self._le_tag('//sig:Signature', 'Id') or ''
            self.URI = self._le_tag('//sig:Signature/sig:SignedInfo/sig:Reference', 'URI') or ''
            self.DigestValue = self._le_tag(
                '//sig:Signature/sig:SignedInfo/sig:Reference/sig:DigestValue') or ''
            self.SignatureValue = self._le_tag('//sig:Signature/sig:SignatureValue') or ''
            self.X509Certificate = self._le_tag(
                '//sig:Signature/sig:KeyInfo/sig:X509Data/sig:X509Certificate') or ''

class tcCpfCnpj(XMLNFSe):
    """
    Número de CPF ou CNPJ
    """
    def __init__(self, raiz=''):
        super(tcCpfCnpj, self).__init__()
        self.nome_tag = 'CpfCnpj'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Cpf = tsCpf(nome='Cpf', codigo='Cpf', raiz=str_raiz, obrigatorio=False)
        self.Cnpj = tsCnpj(nome='Cnpj', codigo='Cnpj', raiz=str_raiz, obrigatorio=False)

    @property
    def xml(self):
        if not self.Cpf.valor and not self.Cnpj.valor:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        if self.Cpf.valor:
            xml.append(self.Cpf.xml)
        else:
            xml.append(self.Cnpj.xml)

        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Cpf.xml = arquivo
            self.Cnpj.xml = arquivo


class tcEndereco(XMLNFSe):
    """
    Representação completa do endereço
    """
    def __init__(self, raiz=''):
        super(tcEndereco, self).__init__()
        self.nome_tag = 'Endereco'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Endereco = tsEndereco(nome='Endereco', codigo='Endereco', raiz=str_raiz,
                                   obrigatorio=False)
        self.Numero = tsNumeroEndereco(nome='Numero', codigo='Numero', raiz=str_raiz,
                                       obrigatorio=False)
        self.Complemento = tsComplementoEndereco(nome='Complemento', codigo='Complemento',
                                                 raiz=str_raiz, obrigatorio=False)
        self.Bairro = tsBairro(nome='Bairro', codigo='Bairro', raiz=str_raiz, obrigatorio=False)
        self.CodigoMunicipio = tsCodigoMunicipioIbge(nome='CodigoMunicipio',
                                                     codigo='CodigoMunicipio', raiz=str_raiz,
                                                     obrigatorio=False)
        self.Uf = tsUf(nome='Uf', codigo='Uf', raiz=str_raiz, obrigatorio=False)
        self.CodigoPais = tsCodigoPaisBacen(nome='CodigoPais', codigo='CodigoPais', raiz=str_raiz,
                                            obrigatorio=False)
        self.Cep = tsCep(nome='Cpf', codigo='Cpf', raiz=str_raiz, obrigatorio=False)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.Endereco.xml)
        xml.append(self.Numero.xml)
        xml.append(self.Complemento.xml)
        xml.append(self.Bairro.xml)
        xml.append(self.CodigoMunicipio.xml)
        xml.append(self.Uf.xml)
        xml.append(self.CodigoPais.xml)
        xml.append(self.Cep.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Endereco.xml = arquivo
            self.Numero.xml = arquivo
            self.Complemento.xml = arquivo
            self.Bairro.xml = arquivo
            self.CodigoMunicipio.xml = arquivo
            self.Uf.xml = arquivo
            self.CodigoPais.xml = arquivo
            self.Cep.xml = arquivo


class tcContato(XMLNFSe):
    """
    Representa forma de contato com a pessoa (física/jurídica)
    """
    def __init__(self, raiz=''):
        super(tcContato, self).__init__()
        self.nome_tag = 'Contato'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Telefone = tsTelefone(nome='Telefone', codigo='Telefone', raiz=str_raiz,
                                   obrigatorio=False)
        self.Email = tsEmail(nome='Email', codigo='Email', raiz=str_raiz, obrigatorio=False)

    @property
    def xml(self):
        if not self.Telefone or not self.Email:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.Telefone.xml)
        xml.append(self.Email.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Telefone.xml = arquivo
            self.Email.xml = arquivo


class tcIdentificacaoOrgaoGerador(XMLNFSe):
    """
    Representa dados para identificação de órgão gerador
    """
    def __init__(self, raiz=''):
        super(tcIdentificacaoOrgaoGerador, self).__init__()
        self.nome_tag = 'OrgaoGerador'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CodigoMunicipio = tsCodigoMunicipioIbge(nome='CodigoMunicipio',
                                                     codigo='CodigoMunicipio', raiz=str_raiz)
        self.Uf = tsUf(nome='Uf', codigo='Uf', raiz=str_raiz)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.CodigoMunicipio.xml)
        xml.append(self.Uf.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CodigoMunicipio.xml = arquivo
            self.Uf.xml = arquivo


class tcIdentificacaoRps(XMLNFSe):
    """
    Dados de identificação do RPS
    """
    def __init__(self, nome, raiz=''):
        super(tcIdentificacaoRps, self).__init__()
        self.nome_tag = nome
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Numero = tsNumeroRps(nome='Numero', codigo='Numero', raiz=str_raiz)
        self.Serie = tsSerieRps(nome='Serie', codigo='Serie', raiz=str_raiz)
        self.Tipo = tsTipoRps(nome='Tipo', codigo='Tipo', raiz=str_raiz)

    @property
    def xml(self):
        if not self.Numero.valor:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.Numero.xml)
        xml.append(self.Serie.xml)
        xml.append(self.Tipo.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Numero.xml = arquivo
            self.Serie.xml = arquivo
            self.Tipo.xml = arquivo


class tcIdentificacaoPrestador(XMLNFSe):
    """
    Representa dados para identificação do prestador de serviço
    """
    def __init__(self, nome, raiz=''):
        super(tcIdentificacaoPrestador, self).__init__()
        self.nome_tag = nome
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome='InscricaoMunicipal',
                                                       codigo='InscricaoMunicipal',
                                                       raiz=str_raiz, obrigatorio=False)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo


class tcIdentificacaoTomador(XMLNFSe):
    """
    Representa dados para identificação do tomador de serviço
    """
    def __init__(self, nome, raiz=''):
        super(tcIdentificacaoTomador, self).__init__()
        self.nome_tag = nome
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome='InscricaoMunicipal',
                                                       codigo='InscricaoMunicipal',
                                                       raiz=str_raiz, obrigatorio=False)

    @property
    def xml(self):
        if not self.CpfCnpj.xml:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo


class tcIdentificacaoConsulente(XMLNFSe):
    """
    Representa dados para identificação do prestador de serviço
    """
    def __init__(self, raiz=''):
        super(tcIdentificacaoConsulente, self).__init__()
        self.nome_tag = 'Consulente'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome='InscricaoMunicipal',
                                                       codigo='InscricaoMunicipal',
                                                       raiz=str_raiz, obrigatorio=False)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo


class tcIdentificacaoIntermediario(XMLNFSe):
    """
    Representa dados para identificação do tomador de serviço
    """
    def __init__(self, nome, raiz=''):
        super(tcIdentificacaoIntermediario, self).__init__()
        self.nome_tag = nome
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome='InscricaoMunicipal',
                                                       codigo='InscricaoMunicipal',
                                                       raiz=str_raiz, obrigatorio=False)

    @property
    def xml(self):
        if not self.CpfCnpj.xml:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo


class tcDadosTomador(XMLNFSe):
    """
    Representa dados do tomador de serviço
    """
    def __init__(self, raiz=''):
        super(tcDadosTomador, self).__init__()
        self.nome_tag = 'Tomador'  # Todo: Na documentacao a tag eh 'TomadorServico' diferente do xsd
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.IdentificacaoTomador = tcIdentificacaoTomador(nome='IdentificacaoTomador',
                                                           raiz=str_raiz)
        self.RazaoSocial = tsRazaoSocial(nome='RazaoSocial', codigo='RazaoSocial',
                                         raiz=str_raiz, obrigatorio=False)
        self.Endereco = tcEndereco(raiz=str_raiz)
        self.Contato = tcContato(raiz=str_raiz)

    @property
    def xml(self):
        if not self.IdentificacaoTomador.xml:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.IdentificacaoTomador.xml)
        xml.append(self.RazaoSocial.xml)
        xml.append(self.Endereco.xml)
        xml.append(self.Contato.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.IdentificacaoTomador.xml = arquivo
            self.RazaoSocial.xml = arquivo
            self.Endereco.xml = arquivo
            self.Contato.xml = arquivo


class tcDadosIntermediario(XMLNFSe):
    """
    Representa dados para identificação de intermediário do serviço
    """
    def __init__(self, raiz=''):
        super(tcDadosIntermediario, self).__init__()
        self.nome_tag = 'Intermediario'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.IdentificacaoIntermediario = tcIdentificacaoIntermediario(
                                                  nome='IdentificacaoIntermediario', raiz=str_raiz)
        self.RazaoSocial = tsRazaoSocial(nome='RazaoSocial', codigo='RazaoSocial', raiz=str_raiz)

    @property
    def xml(self):
        if not self.IdentificacaoIntermediario.xml:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.IdentificacaoIntermediario.xml)
        xml.append(self.RazaoSocial.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.IdentificacaoIntermediario.xml = arquivo
            self.RazaoSocial.xml = arquivo


class tcValoresDeclaracaoServico(XMLNFSe):
    """
    Representa um conjunto de valores que compõe a declaração do serviço
    """
    def __init__(self, raiz=''):
        super(tcValoresDeclaracaoServico, self).__init__()
        self.nome_tag = 'Valores'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.ValorServicos = tsValor(nome='ValorServicos', codigo='ValorServicos', raiz=str_raiz)
        self.ValorDeducoes = tsValor(nome='ValorDeducoes', codigo='ValorDeducoes', raiz=str_raiz,
                                     obrigatorio=False)
        self.ValorPis = tsValor(nome='ValorPis', codigo='ValorPis', raiz=str_raiz,
                                obrigatorio=False)
        self.ValorCofins = tsValor(nome='ValorCofins', codigo='ValorCofins', raiz=str_raiz,
                                   obrigatorio=False)
        self.ValorInss = tsValor(nome='ValorInss', codigo='ValorInss', raiz=str_raiz,
                                         obrigatorio=False)
        self.ValorIr = tsValor(nome='ValorIr', codigo='ValorIr', raiz=str_raiz,
                                         obrigatorio=False)
        self.ValorCsll = tsValor(nome='ValorCsll', codigo='ValorCsll', raiz=str_raiz,
                                         obrigatorio=False)
        self.OutrasRetencoes = tsValor(nome='OutrasRetencoes', codigo='OutrasRetencoes',
                                       raiz=str_raiz, obrigatorio=False)
        self.ValorIss = tsValor(nome='ValorIss', codigo='ValorIss', raiz=str_raiz,
                                         obrigatorio=False)
        self.Aliquota = tsAliquota(nome='Aliquota', codigo='Aliquota', raiz=str_raiz,
                                         obrigatorio=False)
        self.DescontoIncondicionado = tsValor(nome='DescontoIncondicionado',
                                              codigo='DescontoIncondicionado', raiz=str_raiz,
                                              obrigatorio=False)
        self.DescontoCondicionado = tsValor(nome='DescontoCondicionado',
                                            codigo='DescontoCondicionado', raiz=str_raiz,
                                            obrigatorio=False)

    @property
    def xml(self):
        if not self.ValorServicos.valor:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.ValorServicos.xml)
        xml.append(self.ValorDeducoes.xml)
        xml.append(self.ValorPis.xml)
        xml.append(self.ValorCofins.xml)
        xml.append(self.ValorInss.xml)
        xml.append(self.ValorIr.xml)
        xml.append(self.ValorCsll.xml)
        xml.append(self.OutrasRetencoes.xml)
        xml.append(self.ValorIss.xml)
        xml.append(self.Aliquota.xml)
        xml.append(self.DescontoIncondicionado.xml)
        xml.append(self.DescontoCondicionado.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.ValorServicos.xml = arquivo
            self.ValorDeducoes.xml = arquivo
            self.ValorPis.xml = arquivo
            self.ValorCofins.xml = arquivo
            self.ValorInss.xml = arquivo
            self.ValorIr.xml = arquivo
            self.ValorCsll.xml = arquivo
            self.OutrasRetencoes.xml = arquivo
            self.ValorIss.xml = arquivo
            self.Aliquota.xml = arquivo
            self.DescontoIncondicionado.xml = arquivo
            self.DescontoCondicionado.xml = arquivo


class tcValoresNfse(XMLNFSe):
    """
    Representa um conjunto de valores que compõe o documento fiscal
    """
    def __init__(self, raiz=''):
        super(tcValoresNfse, self).__init__()
        self.nome_tag = 'ValoresNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.BaseCalculo = tsValor(nome='BaseCalculo', codigo='BaseCalculo', raiz=str_raiz,
                                   obrigatorio=False)
        self.Aliquota = tsAliquota(nome='Aliquota', codigo='Aliquota', raiz=str_raiz,
                                   obrigatorio=False)
        self.ValorIss = tsValor(nome='ValorIss', codigo='ValorIss', raiz=str_raiz,
                                obrigatorio=False)
        self.ValorLiquidoNfse = tsValor(nome='ValorLiquidoNfse', codigo='ValorLiquidoNfse',
                                        raiz=str_raiz) # valor = (ValorServicos - ValorPIS -
                                                       #          ValorCOFINS - ValorINSS -
                                                       #          ValorIR - ValorCSLL -
                                                       #          OutrasRetençoes -
                                                       #          ValorISSRetido -
                                                       #          DescontoIncondicionado -
                                                       #          DescontoCondicionado)

    @property
    def xml(self):
        if not self.ValorLiquidoNfse.valor:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.BaseCalculo.xml)
        xml.append(self.Aliquota.xml)
        xml.append(self.ValorIss.xml)
        xml.append(self.ValorLiquidoNfse.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.BaseCalculo.xml = arquivo
            self.Aliquota.xml = arquivo
            self.ValorIss.xml = arquivo
            self.ValorLiquidoNfse.xml = arquivo


class tcDadosServico(XMLNFSe):
    """
    Representa dados que compõe o serviço prestado
    """
    def __init__(self, raiz=''):
        super(tcDadosServico, self).__init__()
        self.nome_tag = 'Servico'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Valores = tcValoresDeclaracaoServico(raiz=str_raiz)
        self.IssRetido = tsSimNao(nome='IssRetido', codigo='IssRetido', raiz=str_raiz)
        self.ResponsavelRetencao = tsResponsavelRetencao(nome='ResponsavelRetencao',
                                                         codigo='ResponsavelRetencao',
                                                         raiz=str_raiz, obrigatorio=False)
        self.ItemListaServico = tsItemListaServico(nome='ItemListaServico',
                                                   codigo='ItemListaServico', raiz=str_raiz)
        self.CodigoCnae = tsCodigoCnae(nome='CodigoCnae', codigo='CodigoCnae', raiz=str_raiz,
                                       obrigatorio=False)
        self.CodigoTributacaoMunicipio = tsCodigoTributacao(nome='CodigoTributacaoMunicipio',
                              codigo='CodigoTributacaoMunicipio', raiz=str_raiz, obrigatorio=False)
        self.Discriminacao = tsDiscriminacao(nome='Discriminacao', codigo='Discriminacao',
                                             raiz=str_raiz)
        self.CodigoMunicipio = tsCodigoMunicipioIbge(nome='CodigoMunicipio',
                                                     codigo='CodigoMunicipio', raiz=str_raiz)
        self.CodigoPais = tsCodigoPaisBacen(nome='CodigoPais', codigo='CodigoPais', raiz=str_raiz,
                                            obrigatorio=False)
        self.ExigibilidadeISS = tsExigibilidadeIss(nome='ExigibilidadeISS',
                                                   codigo='ExigibilidadeISS', raiz=str_raiz)
        self.MunicipioIncidencia = tsCodigoMunicipioIbge(nome='MunicipioIncidencia',
                                    codigo='MunicipioIncidencia', raiz=str_raiz, obrigatorio=False)
        self.NumeroProcesso = tsNumeroProcesso(nome='NumeroProcesso', codigo='NumeroProcesso',
                                               raiz=str_raiz, obrigatorio=False)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.Valores.xml)
        xml.append(self.IssRetido.xml)
        xml.append(self.ResponsavelRetencao.xml)
        xml.append(self.ItemListaServico.xml)
        xml.append(self.CodigoCnae.xml)
        xml.append(self.CodigoTributacaoMunicipio.xml)
        xml.append(self.Discriminacao.xml)
        xml.append(self.CodigoMunicipio.xml)
        xml.append(self.CodigoPais.xml)
        xml.append(self.ExigibilidadeISS.xml)
        xml.append(self.MunicipioIncidencia.xml)
        xml.append(self.NumeroProcesso.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Valores.xml = arquivo
            self.IssRetido.xml = arquivo
            self.ResponsavelRetencao.xml = arquivo
            self.ItemListaServico.xml = arquivo
            self.CodigoCnae.xml = arquivo
            self.CodigoTributacaoMunicipio.xml = arquivo
            self.Discriminacao.xml = arquivo
            self.CodigoMunicipio.xml = arquivo
            self.CodigoPais.xml = arquivo
            self.ExigibilidadeISS.xml = arquivo
            self.MunicipioIncidencia.xml = arquivo
            self.NumeroProcesso.xml = arquivo


class tcDadosConstrucaoCivil(XMLNFSe):
    """
    Representa dados para identificação de construção civil
    """
    def __init__(self, raiz=''):
        super(tcDadosConstrucaoCivil, self).__init__()
        self.nome_tag = 'ConstrucaoCivil'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CodigoObra = tsCodigoObra(nome='CodigoObra', codigo='CodigoObra', raiz=str_raiz,
                                       obrigatorio=False)
        self.Art = tsArt(nome='Art', codigo='Art', raiz=str_raiz)

    @property
    def xml(self):
        if not self.Art.valor:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.CodigoObra.xml)
        xml.append(self.Art.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CodigoObra.xml = arquivo
            self.Art.xml = arquivo


class tcDadosPrestador(XMLNFSe):
    """
    Representa dados do prestador do serviço
    """
    def __init__(self, raiz=''):
        super(tcDadosPrestador, self).__init__()
        self.nome_tag = 'PrestadorServico'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.IdentificacaoPrestador = tcIdentificacaoPrestador(nome='IdentificacaoPrestador',
                                                               codigo='IdentificacaoPrestador',
                                                               raiz=str_raiz)
        self.RazaoSocial = tsRazaoSocial(nome='RazaoSocial', codigo='RazaoSocial', raiz=str_raiz)
        self.NomeFantasia = tsNomeFantasia(nome='NomeFantasia', codigo='NomeFantasia',
                                           raiz=str_raiz, obrigatorio=False)
        self.Endereco = tcEndereco(raiz=str_raiz)
        self.Contato = tcContato(raiz=str_raiz)

    @property
    def xml(self):
        if not self.IdentificacaoPrestador.valor and not self.RazaoSocial.valor and \
                self.Endereco.xml:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.IdentificacaoPrestador.xml)
        xml.append(self.RazaoSocial.xml)
        xml.append(self.NomeFantasia.xml)
        xml.append(self.Endereco.xml)
        xml.append(self.Contato.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.IdentificacaoPrestador.xml = arquivo
            self.RazaoSocial.xml = arquivo
            self.NomeFantasia.xml = arquivo
            self.Endereco.xml = arquivo
            self.Contato.xml = arquivo


class tcInfRps(XMLNFSe):
    """
    Representa dados informativos do Recibo Provisório de Serviço (RPS)
    """
    def __init__(self, raiz=''):
        super(tcInfRps, self).__init__()
        self.nome_tag = 'Rps'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome='Rps', codigo='Id', propriedade='Id', raiz=self.raiz_tag)
        self.IdentificacaoRps = tcIdentificacaoRps(nome='IdentificacaoRps', raiz=str_raiz)
        self.DataEmissao = TagData(nome='DataEmissao', codigo='DataEmissao', raiz=str_raiz)
        self.Status = tsStatusRps(nome='Status', codigo='Status', raiz=str_raiz)
        self.RpsSubstituido = tcIdentificacaoRps(nome='RpsSubstituido', raiz=str_raiz)

    @property
    def xml(self):
        if not self.IdentificacaoRps.Numero.valor and not self.DataEmissao.valor and \
                self.Status.valor:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.IdentificacaoRps.xml)
        xml.append(self.DataEmissao.xml)
        xml.append(self.Status.xml)
        if self.RpsSubstituido.xml:
            xml.append(self.RpsSubstituido.xml)

        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.IdentificacaoRps.xml = arquivo
            self.DataEmissao.xml = arquivo
            self.Status.xml = arquivo
            self.RpsSubstituido.xml = arquivo


class tcInfDeclaracaoPrestacaoServico(XMLNFSe):
    """
    Representa dados do da declaração do prestador do serviço
    """
    def __init__(self, raiz=''):
        super(tcInfDeclaracaoPrestacaoServico, self).__init__()
        self.nome_tag = 'InfDeclaracaoPrestacaoServico'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome='InfDeclaracaoPrestacaoServico',
                              codigo='Id', propriedade='Id', raiz=self.raiz_tag)
        self.Rps = tcInfRps(raiz=str_raiz)
        self.Competencia = TagData(nome='Competencia', codigo='Competencia', raiz=str_raiz)
        self.Servico = tcDadosServico(raiz=str_raiz)
        self.Prestador = tcIdentificacaoPrestador(nome='Prestador', raiz=str_raiz)
        self.Tomador = tcDadosTomador(raiz=str_raiz)
        self.Intermediario = tcDadosIntermediario(raiz=str_raiz)
        self.ConstrucaoCivil = tcDadosConstrucaoCivil(raiz=str_raiz)
        self.RegimeEspecialTributacao = tsRegimeEspecialTributacao(nome='RegimeEspecialTributacao',
                               codigo='RegimeEspecialTributacao', raiz=str_raiz,
                                                                   obrigatorio=False)
        self.OptanteSimplesNacional = tsSimNao(nome='OptanteSimplesNacional',
                                               codigo='OptanteSimplesNacional', raiz=str_raiz)
        self.IncentivoFiscal = tsSimNao(nome='IncentivoFiscal', codigo='IncentivoFiscal',
                                        raiz=str_raiz)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.Rps.xml)
        xml.append(self.Competencia.xml)
        xml.append(self.Servico.xml)
        xml.append(self.Prestador.xml)
        xml.append(self.Tomador.xml)
        xml.append(self.Intermediario.xml)
        if self.ConstrucaoCivil:
            xml.append(self.ConstrucaoCivil.xml)

        xml.append(self.RegimeEspecialTributacao.xml)
        xml.append(self.OptanteSimplesNacional.xml)
        xml.append(self.IncentivoFiscal.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.Rps.xml = arquivo
            self.Competencia.xml = arquivo
            self.Servico.xml = arquivo
            self.Prestador.xml = arquivo
            self.Tomador.xml = arquivo
            self.Intermediario.xml = arquivo
            self.ConstrucaoCivil.xml = arquivo
            self.RegimeEspecialTributacao.xml = arquivo
            self.OptanteSimplesNacional.xml = arquivo
            self.IncentivoFiscal.xml = arquivo


class tcDeclaracaoPrestacaoServico(XMLNFSe):
    """
    Representa a estrutura da declaração da prestação do serviço assinada
    """
    def __init__(self, nome='Rps', raiz=''):
        super(tcDeclaracaoPrestacaoServico, self).__init__()
        self.nome_tag = nome
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.InfDeclaracaoPrestacaoServico = tcInfDeclaracaoPrestacaoServico(raiz=str_raiz)
        self.Signature = Signature()

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.InfDeclaracaoPrestacaoServico.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml.append(self.Signature.xml)

        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.InfDeclaracaoPrestacaoServico.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)


class tcIdentificacaoNfse(XMLNFSe):
    """
    Representa dados que identificam uma Nota Fiscal de Serviços Eletrônica
    """
    def __init__(self, raiz=''):
        super(tcIdentificacaoNfse, self).__init__()
        self.nome_tag = 'IdentificacaoNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Numero = tsNumeroNfse(nome='Numero', codigo='Numero', raiz=str_raiz)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome='InscricaoMunicipal',
                                                       codigo='InscricaoMunicipal',
                                                       raiz=str_raiz, obrigatorio=False)
        self.CodigoMunicipio = tsCodigoMunicipioIbge(nome='CodigoMunicipio',
                                                     codigo='CodigoMunicipio', raiz=str_raiz)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.Numero.xml)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append(self.CodigoMunicipio.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Numero.xml = arquivo
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo
            self.CodigoMunicipio.xml = arquivo


class tcInfNfse(XMLNFSe):
    """
    Representa os dados informativos da Nota Fiscal de Serviços Eletrônica
    """
    def __init__(self, raiz=''):
        super(tcInfNfse, self).__init__()
        self.nome_tag = 'InfNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome='NFE', codigo='Id', propriedade='Id', raiz=str_raiz)
        self.Numero = tsNumeroNfse(nome='Numero', codigo='Numero', raiz=str_raiz)
        self.CodigoVerificacao = tsCodigoVerificacao(nome='CodigoVerificacao',
                                                     codigo='CodigoVerificacao', raiz=str_raiz)
        self.DataEmissao = TagDataHora(nome='DataEmissao', codigo='DataEmissao', raiz=str_raiz)
        self.NfseSubstituida = tsNumeroNfse(nome='NfseSubstituida', codigo='NfseSubstituida',
                                            raiz=str_raiz, obrigatorio=False)
        self.OutrasInformacoes = tsOutrasInformacoes(nome='OutrasInformacoes',
                                                     codigo='OutrasInformacoes', raiz=str_raiz,
                                                     obrigatorio=False)
        self.ValoresNfse = tcValoresNfse(codigo='ValoresNfse', raiz=str_raiz)
        self.ValorCredito = tsValor(nome='ValorCredito', codigo='ValorCredito',
                                            raiz=str_raiz, obrigatorio=False)
        self.PrestadorServico = tcDadosPrestador(nome='PrestadorServico',
                                                 codigo='PrestadorServico', raiz=str_raiz)
        self.OrgaoGerador = tcIdentificacaoOrgaoGerador(codigo='OrgaoGerador', raiz=str_raiz)
        self.DeclaracaoPrestacaoServico = tcDeclaracaoPrestacaoServico(
                           nome='DeclaracaoPrestacaoServico', codigo='DeclaracaoPrestacaoServico',
                           raiz=str_raiz)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.Numero.xml)
        xml.append(self.CodigoVerificacao.xml)
        xml.append(self.DataEmissao.xml)
        xml.append(self.NfseSubstituida.xml)
        xml.append(self.OutrasInformacoes.xml)
        xml.append(self.ValoresNfse.xml)
        xml.append(self.ValorCredito.xml)
        xml.append(self.PrestadorServico.xml)
        xml.append(self.OrgaoGerador.xml)
        xml.append(self.DeclaracaoPrestacaoServico.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.Numero.xml = arquivo
            self.CodigoVerificacao.xml = arquivo
            self.DataEmissao.xml = arquivo
            self.NfseSubstituida.xml = arquivo
            self.OutrasInformacoes.xml = arquivo
            self.ValoresNfse.xml = arquivo
            self.ValorCredito.xml = arquivo
            self.PrestadorServico.xml = arquivo
            self.OrgaoGerador.xml = arquivo
            self.DeclaracaoPrestacaoServico.xml = arquivo


class tcNfse(XMLNFSe):
    """
    Representa a estrutura da Nota Fiscal de Serviços Eletrônica assinada
    """
    def __init__(self, raiz=''):
        super(tcNfse, self).__init__()
        self.nome_tag = 'Nfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.versao = TagDecimal(nome='Nfse', codigo='Nfse', propriedade='versao',
                                 raiz=str_raiz, valor='1.00', namespace=NAMESPACE_NFSE)
        self.InfNfse = tcInfNfse(codigo='InfNfse', raiz=str_raiz)
        self.Signature = Signature()

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s versao="%s">' % (self.nome_tag, self.versao.valor))
        xml.append(self.InfNfse.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml.append(self.Signature.xml)

        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.InfNfse.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)


class tcInfPedidoCancelamento(XMLNFSe):
    """
    Representa a estrutura de dados do pedido de cancelamento enviado pelo prestador ao cancelar
     uma Nota Fiscal de Serviços Eletrônica.
    """
    def __init__(self, raiz=''):
        super(tcInfPedidoCancelamento, self).__init__()
        self.nome_tag = 'InfPedidoCancelamento'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome='NFE', codigo='Id', propriedade='Id', raiz=str_raiz)
        self.IdentificacaoNfse = tcIdentificacaoNfse(codigo='IdentificacaoNfse', raiz=str_raiz)
        self.CodigoCancelamento = tsCodigoCancelamentoNfse(nome='CodigoCancelamento',
                                                           codigo='CodigoCancelamento',
                                                           raiz=str_raiz, obrigatorio=False)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.IdentificacaoNfse.xml)
        xml.append(self.CodigoCancelamento.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.IdentificacaoNfse.xml = arquivo
            self.CodigoCancelamento.xml = arquivo


class tcPedidoCancelamento(XMLNFSe):
    """
    Representa a estrutura de Pedido de Cancelamento da Nota Fiscal de Serviços Eletrônica assinada
    """
    def __init__(self, raiz=''):
        super(tcPedidoCancelamento, self).__init__()
        self.nome_tag = 'Pedido'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.InfPedidoCancelamento = tcInfPedidoCancelamento(raiz=str_raiz)
        self.Signature = Signature()

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.InfPedidoCancelamento.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml.append(self.Signature.xml)

        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.InfPedidoCancelamento.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)


class tcConfirmacaoCancelamento(XMLNFSe):
    """
    Representa a estrutura de Confirmação de Cancelamento da Nota Fiscal de Serviços Eletrônica 
    assinada
    """
    def __init__(self, raiz=''):
        super(tcConfirmacaoCancelamento, self).__init__()
        self.nome_tag = 'Confirmacao'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome='CONf', codigo='Id', propriedade='Id', raiz=str_raiz)
        self.Pedido = tcPedidoCancelamento(codigo='Pedido', raiz=str_raiz)
        self.DataHora = TagDataHora(nome='DataHora', codigo='DataHora', raiz=str_raiz)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.Pedido.xml)
        xml.append(self.DataHora.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.Pedido.xml = arquivo
            self.DataHora.xml = arquivo


class tcCancelamentoNfse(XMLNFSe):
    """
    Representa a estrutura completa (pedido + confirmação) de cancelamento de NFS-e
    """
    def __init__(self, raiz=''):
        super(tcCancelamentoNfse, self).__init__()
        self.nome_tag = 'NfseCancelamento'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.versao = TagDecimal(nome='NfseCancelamento', codigo='versao', propriedade='versao',
                                 raiz=str_raiz, valor='1.00', namespace=NAMESPACE_NFSE)
        self.InfNfse = tcInfNfse(codigo='InfNfse', raiz=str_raiz)
        self.Signature = Signature()

    @property
    def xml(self):
        if not self.InfNfse.valor:
            return ''

        xml = [XMLNFSe.get_xml(self)]

        xml.append('<%s versao="%s">' % (self.nome_tag, self.versao.valor))
        xml.append(self.InfNfse.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml.append(self.Signature.xml)

        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.InfNfse.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)


class tcRetCancelamento(XMLNFSe):
    """
    Representa a estrutura de Confirmação de Cancelamento da Nota Fiscal de Serviços Eletrônica 
    assinada
    """
    def __init__(self, raiz=''):
        super(tcRetCancelamento, self).__init__()
        self.nome_tag = 'RetCancelamento'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.NfseCancelamento = tcCancelamentoNfse(codigo='NfseCancelamento', raiz=str_raiz)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]

        xml.append('<%s>' % self.nome_tag)
        xml.append(self.NfseCancelamento.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NfseCancelamento.xml = arquivo


class tcInfSubstituicaoNfse(XMLNFSe):
    """
    Representa os dados de registro de substituição de NFS-e.
    """
    def __init__(self, raiz=''):
        super(tcInfSubstituicaoNfse, self).__init__()
        self.nome_tag = 'SubstituicaoNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome='SubsNfse', codigo='Id', propriedade='Id', raiz=str_raiz)
        self.NfseSubstituidora = tsNumeroNfse(nome='NfseSubstituidora',
                                              codigo='NfseSubstituidora', raiz=str_raiz)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.NfseSubstituidora.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.NfseSubstituidora.xml = arquivo


class tcSubstituicaoNfse(XMLNFSe):
    """
    Representa a estrutura de substituição de NFS-e.
    """
    def __init__(self, raiz=''):
        super(tcSubstituicaoNfse, self).__init__()
        self.nome_tag = 'NfseSubstituicao'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.versao = TagDecimal(nome='NfseCancelamento', codigo='versao', propriedade='versao',
                                 raiz=str_raiz, valor='1.00', namespace=NAMESPACE_NFSE)
        self.SubstituicaoNfse = tcInfSubstituicaoNfse(codigo='SubstituicaoNfse', raiz=str_raiz)
        self.Signature = Signature()

    @property
    def xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append('<%s versao="%s">' % (self.nome_tag, self.versao.valor))
        xml.append(self.SubstituicaoNfse.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml.append(self.Signature.xml)

        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.SubstituicaoNfse.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)


class tcCompNfse(XMLNFSe):
    """
    Representa a estrutura de compartilhamento de dados de uma NFS-e.
    """

    def __init__(self, raiz=''):
        super(tcCompNfse, self).__init__()
        self.nome_tag = 'CompNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Nfse = tcNfse(codigo='Nfse', raiz=str_raiz)
        self.NfseCancelamento = tcCancelamentoNfse(codigo='NfseCancelamento', raiz=str_raiz,
                                                   obrigatorio=False)
        self.NfseSubstituicao = tcSubstituicaoNfse(codigo='NfseSubstituicao', raiz=str_raiz,
                                                   obrigatorio=False)

    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]

        xml.append('<%s>' % self.nome_tag)
        xml.append(self.Nfse.xml)
        xml.append(self.NfseCancelamento.xml)
        xml.append(self.NfseSubstituicao.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Nfse.xml = arquivo
            self.NfseCancelamento.xml = arquivo
            self.NfseSubstituicao.xml = arquivo


class tcMensagemRetorno(XMLNFSe):
    """
    Representa a estrutura de mensagem de retorno de serviço.
    """

    def __init__(self, raiz=''):
        super(tcMensagemRetorno, self).__init__()
        self.nome_tag = 'MensagemRetorno'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Codigo = tsCodigoMensagemAlerta(nome='Codigo', codigo='Codigo', raiz=str_raiz)
        self.Mensagem = tsDescricaoMensagemAlerta(nome='Mensagem', codigo='Mensagem',
                                                  raiz=str_raiz)
        self.Correcao = tsDescricaoMensagemAlerta(nome='Correcao', codigo='Correcao',
                                                  raiz=str_raiz, obrigatorio=False)

    @property
    def xml(self):
        if not self.Mensagem.valor:
            return ''

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<%s>' % self.nome_tag)
        xml.append(self.Codigo.xml)
        xml.append(self.Mensagem.xml)
        xml.append(self.Correcao.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Codigo.xml = arquivo
            self.Mensagem.xml = arquivo
            self.Correcao.xml = arquivo


class tcMensagemRetornoLote(XMLNFSe):
    """
    Representa a estrutura de mensagem de retorno de serviço.
    """

    def __init__(self, raiz=''):
        super(tcMensagemRetornoLote, self).__init__()
        self.nome_tag = 'MensagemRetorno'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.IdentificacaoRps = tcIdentificacaoRps(nome='IdentificacaoRps',
                                                   codigo='IdentificacaoRps', raiz=str_raiz)
        self.Codigo = tsCodigoMensagemAlerta(nome='Codigo', codigo='Codigo', raiz=str_raiz)
        self.Mensagem = tsDescricaoMensagemAlerta(nome='Mensagem', codigo='Mensagem',
                                                  raiz=str_raiz)

    @property
    def xml(self):
        if not self.Codigo.valor and not self.Mensagem.valor:
            return ''

        xml = [XMLNFSe.get_xml(self)]

        xml.append('<%s>' % self.nome_tag)
        if self.IdentificacaoRps.xml:
            xml.append(self.IdentificacaoRps.xml)

        xml.append(self.Codigo.xml)
        xml.append(self.Mensagem.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.IdentificacaoRps.xml = arquivo
            self.Codigo.xml = arquivo
            self.Mensagem.xml = arquivo


class tcLoteRps(XMLNFSe):
    """
    Representa a estrutura do lote de RPS para fila de processamento
    """
    def __init__(self, raiz=''):
        super(tcLoteRps, self).__init__()
        self.nome_tag = 'LoteRps'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome='LoteRps', codigo='Id', propriedade='Id', raiz=self.raiz_tag)
        self.versao = TagDecimal(nome='LoteRps', codigo='versao', propriedade='versao',
                                 raiz=self.raiz_tag, valor='1.00', namespace=NAMESPACE_NFSE)
        self.NumeroLote = tsNumeroLote(nome='NumeroLote', codigo='NumeroLote', raiz=str_raiz)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome='InscricaoMunicipal',
                                                       codigo='InscricaoMunicipal',
                                                       raiz=self.raiz_tag,
                                                       obrigatorio=False)
        self.QuantidadeRps = tsQuantidadeRps(nome='QuantidadeRps', codigo='QuantidadeRps',
                                             raiz=str_raiz)
        self.Rps = [] # tcInfDeclaracaoPrestacaoServico

    @property
    def xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append('<%s Id="%s" versao="%s">' % (self.nome_tag, self.Id.valor, self.versao.valor))
        xml.append(self.NumeroLote.xml)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append(self.QuantidadeRps.xml)
        xml.append('<ListaRps>')
        for i in self.Rps:
            xml.append(i.xml)

        xml.append('</ListaRps>')
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.versao.xml = arquivo
            self.NumeroLote.xml = arquivo
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo
            self.QuantidadeRps.xml = arquivo
            self.Rps = self.le_grupo('%s/%s/ListaRps' % (self.raiz_tag, self.nome_tag),
                                     tcDeclaracaoPrestacaoServico, arquivo)


class ListaMensagemRetornoLote(XMLNFSe):
    """
    Representa a estrutura de mensagem de retorno de serviço.
    """
    def __init__(self, raiz=''):
        super(ListaMensagemRetornoLote, self).__init__()
        self.nome_tag = 'ListaMensagemRetornoLote'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.MensagemRetorno = []

    @property
    def xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append('<%s>' % self.nome_tag)
        for i in self.MensagemRetorno:
            xml.append(i.xml)

        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.MensagemRetorno = self.le_grupo('//%s/%s/MensagemRetorno' % (self.raiz_tag,
                                                                              self.nome_tag),
                                                 tcMensagemRetornoLote)


class ListaMensagemRetorno(XMLNFSe):
    """
    Representa a estrutura de mensagem de retorno de serviço.
    """
    def __init__(self, raiz=''):
        super(ListaMensagemRetorno, self).__init__()
        self.nome_tag = 'ListaMensagemRetorno'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.MensagemRetorno = tcMensagemRetorno(raiz=str_raiz)

    @property
    def xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append('<%s>' % self.nome_tag)
        xml.append(self.MensagemRetorno.xml)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.MensagemRetorno.xml = arquivo


class ListaMensagemAlertaRetorno(XMLNFSe):
    """
    Representa a estrutura de mensagem de retorno de serviço.
    """
    def __init__(self, raiz=''):
        super(ListaMensagemAlertaRetorno, self).__init__()
        self.nome_tag = 'ListaMensagemAlertaRetorno'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.MensagemRetorno = tcMensagemRetorno(raiz=str_raiz)

    @property
    def xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append('<%s>' % self.nome_tag)
        xml.append(self.MensagemRetorno.xml)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.MensagemRetorno.xml = arquivo


class CompNfse(XMLNFSe):
    """
    Representa a estrutura da NFS-e.
    """
    def __init__(self, raiz=''):
        super(CompNfse, self).__init__()
        self.nome_tag = 'CompNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CompNfse = tcCompNfse(codigo='CompNfse', raiz=str_raiz)

    @property
    def xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append('<%s>' % self.nome_tag)
        xml.append(self.CompNfse.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CompNfse.xml = arquivo




# ----------------------------Servicos---------------------------------


class EnviarLoteRpsEnvio(XMLNFSe):
    def __init__(self, raiz=''):
        super(EnviarLoteRpsEnvio, self).__init__()
        self.nome_tag = 'EnviarLoteRpsEnvio'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.LoteRps = tcLoteRps(raiz=str_raiz)
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/')
        self.arquivo_esquema = 'nfse_v202.xsd'

    @property
    def xml(self):

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<?xml version="1.0"?>')
        xml.append('<%s %s>' % (self.nome_tag, 'xmlns="%s"' % NAMESPACE_NFSE))
        xml.append(self.LoteRps.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml.append(self.Signature.xml)

        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.LoteRps.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)


class EnviarLoteRpsResposta(XMLNFSe):
    def __init__(self, raiz=''):
        super(EnviarLoteRpsResposta, self).__init__()
        self.nome_tag = 'EnviarLoteRpsResposta'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.NumeroLote = tsNumeroLote(nome='NumeroLote', codigo='NumeroLote', raiz=str_raiz)
        self.DataRecebimento = TagDataHora(nome='DataRecebimento', codigo='DataRecebimento',
                                           raiz=str_raiz)
        self.Protocolo = tsNumeroProtocolo(nome='Protocolo', codigo='Protocolo', raiz=str_raiz)
        self.ListaMensagemRetorno = ListaMensagemRetorno(raiz=str_raiz)
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/')
        self.arquivo_esquema = 'nfse_v202.xsd'

    @property
    def xml(self):

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<?xml version="1.0"?>')
        xml.append('<%s %s>' % (self.nome_tag, 'xmlns="%s"' % NAMESPACE_NFSE))
        xml.append(self.NumeroLote.xml)
        xml.append(self.DataRecebimento.xml)
        xml.append(self.Protocolo.xml)
        xml.append(self.ListaMensagemRetorno.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NumeroLote.xml = arquivo
            self.DataRecebimento.xml = arquivo
            self.Protocolo.xml = arquivo
            self.ListaMensagemRetorno.xml = arquivo


class EnviarLoteRpsSincronoEnvio(XMLNFSe):
    def __init__(self, raiz=''):
        super(EnviarLoteRpsSincronoEnvio, self).__init__()
        self.nome_tag = 'EnviarLoteRpsSincronoEnvio'
        self.raiz_tag = raiz
        str_raiz = '//%s%s' % (raiz, self.nome_tag)
        self.LoteRps = tcLoteRps(raiz=str_raiz)
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/')
        self.arquivo_esquema = 'nfse_v202.xsd'


    @property
    def xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append('<?xml version="1.0"?>')
        xml.append('<%s %s>' % (self.nome_tag, 'xmlns="%s"' % NAMESPACE_NFSE))
        xml.append(self.LoteRps.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml.append(self.Signature.xml)

        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.LoteRps.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)


class EnviarLoteRpsSincronoResposta(XMLNFSe):
    def __init__(self, raiz=''):
        super(EnviarLoteRpsSincronoResposta, self).__init__()
        self.nome_tag = 'EnviarLoteRpsSincronoResposta'
        self.raiz_tag = raiz
        str_raiz = '//%s%s' % (raiz, self.nome_tag)
        self.NumeroLote = tsNumeroLote(nome='NumeroLote', codigo='NumeroLote', raiz=str_raiz)
        self.DataRecebimento = TagDataHora(nome='DataRecebimento', codigo='DataRecebimento',
                                           raiz=str_raiz)
        self.Protocolo = tsNumeroProtocolo(nome='Protocolo', codigo='Protocolo', raiz=str_raiz)
        self.CompNfse = []
        self.ListaMensagemAlertaRetorno = ListaMensagemAlertaRetorno(raiz=str_raiz)
        self.ListaMensagemRetorno = ListaMensagemRetorno(raiz=str_raiz)
        self.ListaMensagemRetornoLote = ListaMensagemRetornoLote(raiz=str_raiz)
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/')
        self.arquivo_esquema = 'nfse_v202.xsd'

    @property
    def xml(self):

        xml = [XMLNFSe.get_xml(self)]
        xml.append('<?xml version="1.0"?>')
        xml.append('<%s %s>' % (self.nome_tag, 'xmlns="%s"' % NAMESPACE_NFSE))
        xml.append(self.NumeroLote.xml)
        xml.append(self.DataRecebimento.xml)
        xml.append(self.Protocolo.xml)
        xml.append('<ListaNfse>')
        for i in self.CompNfse:
            xml.append(i.xml)

        xml.append(self.ListaMensagemAlertaRetorno.xml)
        xml.append('</ListaNfse>')
        xml.append(self.ListaMensagemRetorno.xml)
        xml.append(self.ListaMensagemRetornoLote.xml)
        xml.append('</%s>' % self.nome_tag)
        return ''.join(xml)

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NumeroLote.xml = arquivo
            self.DataRecebimento.xml = arquivo
            self.Protocolo.xml = arquivo
            self.ListaNfse = self.le_grupo('%s/%s/ListaNfse' % (self.raiz_tag, self.nome_tag),
                                           tcCompNfse, arquivo)
            self.ListaMensagemRetorno.xml = arquivo
            self.ListaMensagemRetornoLote.xml = arquivo

