# -*- coding: utf-8 -*-

from .base import XMLNFe
import os


DIRNAME = os.path.dirname(__file__)


class Signature(XMLNFe):
    def __init__(self):
        super(Signature, self).__init__()
        self.URI = ''
        self.DigestValue = ''
        self.SignatureValue = ''
        self.X509Certificate = ''
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/')
        self.arquivo_esquema = 'xmldsig-core-schema_v1.01.xsd'

    @property
    def xml(self):
        if not len(self.URI):
            self.URI = '#'
        if self.URI[0] != '#':
            self.URI = '#' + self.URI
        xml = '<Signature xmlns="http://www.w3.org/2000/09/xmldsig#">'
        xml += '<SignedInfo>'
        xml += '<CanonicalizationMethod Algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315"/>'
        xml += '<SignatureMethod Algorithm="http://www.w3.org/2000/09/xmldsig#rsa-sha1"/>'
        xml += '<Reference URI="' + self.URI + '">'
        xml += '<Transforms>'
        xml += '<Transform Algorithm="http://www.w3.org/2000/09/xmldsig#enveloped-signature"/>'
        xml += '<Transform Algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315"/>'
        xml += '</Transforms>'
        xml += '<DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>'
        xml += '<DigestValue>' + self.DigestValue + '</DigestValue>'
        xml += '</Reference>'
        xml += '</SignedInfo>'
        xml += '<SignatureValue>' + self.SignatureValue + '</SignatureValue>'
        xml += '<KeyInfo>'
        xml += '<X509Data>'
        xml += '<X509Certificate>' + self.X509Certificate + '</X509Certificate>'
        xml += '</X509Data>'
        xml += '</KeyInfo>'
        xml += '</Signature>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.URI = self._le_tag('//sig:Signature/sig:SignedInfo/sig:Reference', 'URI') or ''
            self.DigestValue = self._le_tag(
                '//sig:Signature/sig:SignedInfo/sig:Reference/sig:DigestValue') or ''
            self.SignatureValue = self._le_tag('//sig:Signature/sig:SignatureValue') or ''
            self.X509Certificate = self._le_tag(
                '//sig:Signature/sig:KeyInfo/sig:X509Data/sig:X509Certificate') or ''

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
